package analysisOperations;

import java.awt.Rectangle;

import ij.ImagePlus;
import ij.measure.ResultsTable;
import ij.process.ImageProcessor;
import gui.Option;
import gui.Options;
import operations.Operation;

public class MeasureMaxCubeOperation extends Operation {
	/**
	 * 
	 */
	private static final long serialVersionUID = -921442050558400265L;
	
	Option radius;
	
	ResultsTable measurements;

	protected int startX;

	protected int startY;

	protected int endX;

	protected int endY;

	protected int startZ;

	protected int endZ;

	protected int bestX;
	protected int bestY;
	protected int bestZ;

	private int count;
	
	protected void initialize() throws ClassNotFoundException {
		super.initialize();
		parameterTypes = new Class[1];
		parameterTypes[0] = Class.forName("ij.ImagePlus");
		parameterNames = new String[1];
		parameterNames[0] = "InputImage";
		resultTypes = new Class[1];
		resultTypes[0] = Class.forName("ij.measure.ResultsTable");
		resultNames = new String[1];
		resultNames[0] = "Measurements";
		optionsNames = new String[1];
		optionsNames[0] = "radius";
	}

	public void doIt() {
		ImagePlus inputImage = this.getInputImage();
		long max = 0;
		bestX = -1;
		bestY = -1;
		bestZ = -1;
		long currentValue;
		calculateStartAndEndPoints(inputImage);
		for (int x=startX; x<=endX; x++) {
			for (int y=startY; y<=endY; y++) {
				for (int z=startZ; z<=endZ; z++) {
					currentValue = this.calculateIntensity(x,y,z);
					if (currentValue>max) {
						max = currentValue;
						bestX = x;
						bestY = y;
						bestZ = z;
					}
				}
			}
		}
		createResultsTable(max);
		if (this.getShowResult()) this.measurements.show("best cube");
	}

	protected void createResultsTable(long max) {
		this.measurements = new ResultsTable();
		this.measurements.reset();
		this.measurements.setHeading(0,"integrated intensity");
		this.measurements.setHeading(1,"average intensity");
		this.measurements.setHeading(2,"x");
		this.measurements.setHeading(3,"y");
		this.measurements.setHeading(4,"z");
		this.measurements.setHeading(5,"radius");
		this.measurements.incrementCounter();
		this.measurements.addValue(0, max);
		this.measurements.addValue(1, max / (count * 1.0));
		this.measurements.addValue(2, bestX);
		this.measurements.addValue(3, bestY);
		this.measurements.addValue(4, bestZ);
		this.measurements.addValue(5, this.getRadius());
	}

	protected long calculateIntensity(int x, int y, int z) {
		long sum = 0;
		count = 0; 
		int r = this.getRadius();
		for (int cubeX=x-r; cubeX<=x+r; cubeX++) {
			for (int cubeY=y-r; cubeY<=y+r; cubeY++) {
				for (int cubeZ=z-r; cubeZ<=z+r; cubeZ++) {
					count++;
					sum = sum + this.getValueAt(cubeX, cubeY, cubeZ);
				}
			}
		}
		return sum;
	}

	private long getValueAt(int cubeX, int cubeY, int cubeZ) {
		ImagePlus inputImage = this.getInputImage();
		if (cubeZ<1 || cubeZ > inputImage.getImageStackSize()) return 0;
		ImageProcessor ip = inputImage.getImageStack().getProcessor(cubeZ);
		return ip.getPixel(cubeX, cubeY);
	}

	protected void calculateStartAndEndPoints(ImagePlus inputImage) {
		startZ = 1;
		endZ = inputImage.getStackSize();
		startX = 0;
		startY = 0;
		endX = inputImage.getWidth()-1;
		endY = inputImage.getHeight()-1;
		if (inputImage.getRoi()!=null) {
			Rectangle bounds = this.getInputImage().getRoi().getBounds();
			startX = bounds.x;
			startY = bounds.y;
			endX = startX + bounds.width - 1;
			endY = startY + bounds.height - 1;
		}
	}
	
	public ResultsTable getMeasurements() {
		return measurements;
	}

	public void setMeasurements(ResultsTable measurements) {
		this.measurements = measurements;
	}

	public int getRadius() {
		return radius.getIntegerValue();
	}

	public void setRadius(int radius) {
		this.radius.setValue(Integer.toString(radius));
	}

	protected void setupOptions() {
		super.setupOptions();
		this.setRadius(1);
		radius.setShortHelpText("The radius of the cube");
	}
	
	public void connectOptions() {
		radius = (Option) options.getOptions().elementAt(0);
	}
}
