/*
 * Created on 22.08.2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package analysisOperations;

import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.util.Vector;

import gui.LineProcessor;
import gui.Option;
import ij.ImagePlus;
import ij.gui.NewImage;
import ij.measure.ResultsTable;
import operations.Operation;

/**
 * @author Volker
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class TraceLinesOperation extends Operation {
	/**
	 * 
	 */
	private static final long serialVersionUID = -3104046059535469398L;
	
	protected ResultsTable startPoints;
	protected Option minLength;
	protected Option maxAngle;
	protected Option minDistanceBorder;
	protected Option minSegmentLength;
	protected Vector resultLines;
		
	protected void initialize() throws ClassNotFoundException {
		super.initialize();
		parameterTypes = new Class[2];
		parameterTypes[0] = Class.forName("ij.ImagePlus");
		parameterTypes[1] = Class.forName("ij.measure.ResultsTable");
		parameterNames = new String[2];
		parameterNames[0] = "InputImage";
		parameterNames[1] = "StartPoints";
		optionsNames = new String[4];
		optionsNames[0] = "min stick length";
		optionsNames[1] = "max angle";
		optionsNames[2] = "distance border";
		optionsNames[3] = "min segment length";
		resultTypes = new Class[2];
		resultTypes[0] = Class.forName("ij.ImagePlus");
		resultTypes[1] = Class.forName("java.util.Vector");
		resultNames = new String[2];
		resultNames[0] = "Result";
		resultNames[1] = "ResultLines";
	}
	/**
	 * @return Returns the maxAngle.
	 */
	public float getMaxAngle() {
		return maxAngle.getFloatValue();
	}
	/**
	 * @param maxAngle The maxAngle to set.
	 */
	public void setMaxAngle(float maxAngle) {
		this.maxAngle.setValue(Float.valueOf(maxAngle).toString());
	}
	/**
	 * @return Returns the minLength.
	 */
	public float getMinLength() {
		return minLength.getFloatValue();
	}
	/**
	 * @param minLength The minLength to set.
	 */
	public void setMinLength(float minLength) {
		this.minLength.setValue(Float.valueOf(minLength).toString());
	}
	/**
	 * @return Returns the startPoints.
	 */
	public ResultsTable getStartPoints() {
		return startPoints;
	}
	/**
	 * @param startPoints The startPoints to set.
	 */
	public void setStartPoints(ResultsTable startPoints) {
		this.startPoints = startPoints;
	}
	
	protected void setupOptions() {
		super.setupOptions();
		this.setMinLength(60);
		this.setMaxAngle(7);
		this.setMinDistanceBorder(60);
		this.setMinSegmentLength(600);
	}
	
	public void connectOptions() {
		this.minLength = (Option) this.options.getOptions().elementAt(0);
		this.maxAngle = (Option) this.options.getOptions().elementAt(1);
		this.minDistanceBorder = (Option) this.options.getOptions().elementAt(2);
		this.minSegmentLength = (Option) this.options.getOptions().elementAt(3);
	}
	
	protected void cleanUpInput() {
		super.cleanUpInput();
		this.setStartPoints(null);
	}
	/**
	 * @return Returns the resultLines.
	 */
	public Vector getResultLines() {
		return resultLines;
	}
	/**
	 * @param resultLines The resultLines to set.
	 */
	public void setResultLines(Vector resultLines) {
		this.resultLines = resultLines;
	}
	
	public void doIt() {
		result = NewImage.createByteImage(inputImage.getTitle(), inputImage.getWidth(), inputImage.getHeight(), 
				1, NewImage.GRAY8);
		resultLines = new Vector();
		for (int i=0; i<startPoints.getCounter(); i++) {
			float x = startPoints.getValue(ResultsTable.X_CENTROID, i);
			float y = startPoints.getValue(ResultsTable.Y_CENTROID, i);
			Point2D center = new Point2D.Float(x,y);
			Point2D left = this.traceLeft(center);
			Point2D right = this.traceRight(center);
			Line2D aSegment = new Line2D.Float(left,right);
			if (this.alreadyFound(aSegment)) continue;
			if (aSegment.getP1().distance(aSegment.getP2())<this.getMinSegmentLength()) continue;
			if (this.nearBorder(aSegment, result)) continue;
			resultLines.add(aSegment);
		}
		drawLines(resultLines, result.getProcessor());
	}

	/**
	 * @param segment
	 * @param result
	 * @return
	 */
	private boolean nearBorder(Line2D segment, ImagePlus result) {
		Point2D point = new Point2D.Double(segment.getX2(), segment.getY2());
		LineProcessor.movePointIntoDirectionOfBy(point, segment,this.getMinDistanceBorder());
		if (point.getX()>=result.getWidth() ||
			point.getY()>=result.getHeight() ||
			point.getX() < 0 ||
			point.getY() <0) return true;
		point = new Point2D.Double(segment.getX1(), segment.getY1());
		Line2D inversedLine = LineProcessor.inverseLine(segment);
		LineProcessor.movePointIntoDirectionOfBy(point, inversedLine,this.getMinDistanceBorder());
		if (point.getX()>=result.getWidth() ||
				point.getY()>=result.getHeight() ||
				point.getX() < 0 ||
				point.getY() <0) return true;
		return false;
	}

	/**
	 * @param segment
	 * @return
	 */
	private boolean alreadyFound(Line2D segment) {
		if (resultLines.size()==0) return false;
		if (resultLines.contains(segment)) return true;
		Line2D lastLine = (Line2D) resultLines.lastElement();
		if (lastLine.ptLineDist(segment.getP1())<this.minLineDistance() || 
				lastLine.ptLineDist(segment.getP2())<this.minLineDistance()) return true;
		return false;
	}

	public int minLineDistance() {
		return 10;
	}
	/**
	 * @param center
	 * @return
	 */
	private Point2D traceRight(Point2D center) {
		return this.trace(center, true);
	}

	/**
	 * @param center
	 * @return
	 */
	private Point2D traceLeft(Point2D center) {
		return this.trace(center, false);
	}

	private Point2D trace(Point2D center, boolean right) {
		double currentX = center.getX();
		double currentY = center.getY();
		Point2D currentPoint = new Point2D.Double(center.getX(), center.getY());
		double l = this.getMinLength();
		double pi = Math.PI / 180.0;
		while (currentX<inputImage.getWidth() && currentY<inputImage.getHeight() &&
				currentX>0 && currentY>0) {
			double bestAverage = 0;
			double bestEndX = currentX + l;
			if (!right) bestEndX = currentX - l;
			double bestEndY = currentY;
			float turn = 0;
			if (!right) turn = 180; 
			for (float angle = (-this.getMaxAngle() + turn); angle <= (this.getMaxAngle() + turn); angle++) {
				double endX = currentX + (l * Math.cos(angle * (pi)));
				double endY = currentY + (l * Math.sin(angle * (pi)));
				double[] pixelValues = inputImage.getProcessor().getLine(
						currentX, currentY, endX, endY);
				double average = this.getAverage(pixelValues);
				if (average > bestAverage) {
					bestAverage = average;
					bestEndX = endX;
					bestEndY = endY;
				}
			}
			Line2D currentSegment = new Line2D.Double(currentX, currentY,
					bestEndX, bestEndY);
			Line2D perpendicularLine = LineProcessor.perpendicularLine(
					currentSegment, l);
			double[] pixelValues = inputImage.getProcessor().getLine(
					perpendicularLine.getX1(), perpendicularLine.getY1(),
					perpendicularLine.getX2(), perpendicularLine.getY2());
			double threshold = this.getAverage(pixelValues);
			if (bestAverage<=threshold) break;
			LineProcessor.movePointIntoDirectionOf(currentPoint,currentSegment);
			currentX = currentPoint.getX();
			currentY = currentPoint.getY();
		}
		return currentPoint;
	}
	/**
	 * @param pixelValues
	 * @return
	 */
	private double getAverage(double[] pixelValues) {
		double result = 0;
		for (int i=0; i<pixelValues.length; i++) {
			result = result + pixelValues[i];
		}
		result = result / pixelValues.length;
		return result;
	}
	/**
	 * @return Returns the minDistanceBorder.
	 */
	public int getMinDistanceBorder() {
		return minDistanceBorder.getIntegerValue();
	}
	/**
	 * @param minDistanceBorder The minDistanceBorder to set.
	 */
	public void setMinDistanceBorder(int minDistanceBorder) {
		this.minDistanceBorder.setValue(Integer.valueOf(minDistanceBorder).toString());
	}
	/**
	 * @return Returns the minSegmentLength.
	 */
	public float getMinSegmentLength() {
		return minSegmentLength.getFloatValue();
	}
	/**
	 * @param minSegmentLength The minSegmentLength to set.
	 */
	public void setMinSegmentLength(float minSegmentLength) {
		this.minSegmentLength.setValue(Float.valueOf(minSegmentLength).toString());
	}
}
