/*
 * Created on 21.06.2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package gui;

import fileOperations.OpenImageOperation;
import fileOperations.SaveImageOperation;
import ij.ImageJ;
import ij.ImagePlus;
import ij.WindowManager;
import ij.measure.Calibration;
import ij.measure.ResultsTable;
import ij.IJ;
import imageOperations.InvertImageOperation;
import imagejProxies.MRIJava2;

import java.io.File;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Map;
import java.util.Observable;
import java.util.Vector;

import javax.swing.JMenu;
import javax.swing.JMenuItem;

import neuronJNewGUI.NeuronJ;
import neuronJgui.NJ;
import neuronJgui.NeuronJ_;

import analysisOperations.FindObjectsOperation;
import applications.Application;

import channelOperations.SplitChannelsOperation;

import operations.FindStraightLinesOperation;
import operations.Operation;
import particleAnalysis.threeDimensional._3D_objects_counter;
import processingOperations.DilateOperation;
import processingOperations.ErodeOperation;
import processingOperations.SubstractMeanOperation;
import segmentationOperations.AutoThresholdOperation;
import segmentationOperations.EntropyThresholdOperation;
import segmentationOperations.MeanThresholdOperation;
import segmentationOperations.OtsuThresholdOperation;
import movie.MRI_QT_Stack_Writer_fast;

/**
 * @author Volker
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class CellImageAnalyzer extends Observable {
	/** The user interface of the launcher window. The launcher window is the 
	 * applications main window.
	 */
	protected CellImageAnalyzerView view;
	/**
	 * Default Constructor. Initializes the application.
	 *
	 */
	
	Map applicationFiles = new HashMap();
	Map operationFiles = new HashMap();
	
	static protected CellImageAnalyzer current = null;
	
	public CellImageAnalyzer() {
		current = this;
		this.initialize();
		resetScale();
	}
	
	/**
	 * 
	 */
	private void resetScale() {
		ImagePlus image = new ImagePlus();
		Calibration calibration = image.getCalibration();
		calibration.setUnit("pixel");
		image.setGlobalCalibration(calibration);
	}

	/**
	 * Initialize the application. Create the view and register myself as model. 
	 */
	private void initialize() {
		try {
			if (IJ.getInstance()==null) {
				new ImageJ();
				IJ.getInstance().setVisible(false);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		MRIJava2.setLookAndFeelSet(true);
		view = new CellImageAnalyzerView(this);
		view.setModel(this);
	}
	
	/**
	 * Create a new instance and run it.
	 * @param args	not used
	 */
	public static void main(String[] args) {
		CellImageAnalyzer model = new CellImageAnalyzer();
		model.run();
	}
	
	/**
	 * Show my view.
	 */
	private void run() {
		view.setVisible(true);
	}
	
	/**
	 * Notify observers that an aspect of the client has changed.
	 */
	protected void changed(Object anObject) {
		this.setChanged();//required by notifyObservers()
		notifyObservers(anObject);
	}
	
	/**
	 * The open image menu action. Ask the user to select an image file. Open the
	 * image and show it.
	 */
	public void openImage() {
		OpenImageOperation openImageAction = new OpenImageOperation();
		openImageAction.execute();
		ImagePlus image = openImageAction.getResult();
		if (image==null) return;
		image.show();
	}

	/**
	 * 
	 */
	public void splitChannels() {
		SplitChannelsOperation splitChannelsAction = new SplitChannelsOperation();
		ImagePlus inputImage = WindowManager.getCurrentImage();
		if (inputImage == null) return;
		if (inputImage.getType()!=ImagePlus.COLOR_256 && inputImage.getType()!=ImagePlus.COLOR_RGB) {
			return;
		}
		splitChannelsAction.setInputImage(inputImage);
		splitChannelsAction.execute();
	}
	
	/**
	 * 
	 */
	public void autoThreshold() {
		ImagePlus inputImage = WindowManager.getCurrentImage();
		AutoThresholdOperation action = new AutoThresholdOperation();
		action.setInputImage(inputImage);
		action.execute();
	}
	
	/**
	 * 
	 */
	public void entropyThreshold() {
		ImagePlus inputImage = WindowManager.getCurrentImage();
		EntropyThresholdOperation action = new EntropyThresholdOperation();
		action.setInputImage(inputImage);
		action.execute();
	}
	
	/**
	 * 
	 */
	public void meanThreshold() {
		ImagePlus inputImage = WindowManager.getCurrentImage();
		MeanThresholdOperation action = new MeanThresholdOperation();
		action.setInputImage(inputImage);
		action.execute();
	}
	
	/**
	 * 
	 */
	public void otsuThreshold() {
		ImagePlus inputImage = WindowManager.getCurrentImage();
		OtsuThresholdOperation action = new OtsuThresholdOperation();
		action.setInputImage(inputImage);
		action.execute();
	}
	/**
	 * 
	 */
	public void invertImage() {
		ImagePlus inputImage = WindowManager.getCurrentImage();
		InvertImageOperation action = new InvertImageOperation();
		action.setInputImage(inputImage);
		action.execute();
	}
	
	/**
	 * Close the application. Close all image windows before exiting.
	 */
	public void exit() {
		WindowManager.closeAllWindows();
		IJ.getInstance().quit();
		System.exit(0);
	}

	/**
	 * 
	 */
	public void findParticles() {
		ImagePlus inputImage = WindowManager.getCurrentImage();
		FindObjectsOperation operation = new FindObjectsOperation();
		operation.setInputImage(inputImage);
		operation.setMeasurements(ResultsTable.getResultsTable());
		operation.execute();
		ResultsTable measurements = operation.getMeasurements();
		measurements.show(inputImage.getTitle() + " particles");
	}

	/**
	 * 
	 */
	public void findStraightLines() {
		ImagePlus inputImage = WindowManager.getCurrentImage();
		ResultsTable measurements = ResultsTable.getResultsTable();
		FindStraightLinesOperation operation = new FindStraightLinesOperation();
		operation.setInputImage(inputImage);
		operation.setCentroids(measurements);
		operation.setMinLength(200);
		operation.setMinMeanGreyLevel(50);
		operation.execute();
		ImagePlus result = operation.getResult();
		result.show();
	}

	/**
	 * 
	 */
	public void saveAs() {
		ImagePlus inputImage = WindowManager.getCurrentImage();
		SaveImageOperation save = new SaveImageOperation();
		save.setInputImage(inputImage);
		save.execute();
	}

	/**
	 * @param string
	 */
	public void showOperations(String category) {
		OperationsBox box = new OperationsBox();
		box.setTitle(category + " operations");
		Vector classes = Operation.operationClasses();
		for (int i=0; i<classes.size();i++) {
			try {
				Class aClass = (Class) classes.elementAt(i);
				Constructor ct;
				ct = aClass.getConstructor(null);
				Operation operation = (Operation) ct.newInstance(null);
				if (category.equals("all")) {
					box.add(operation);
				}
			} catch (SecurityException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (NoSuchMethodException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalArgumentException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InstantiationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		box.show();
	}
	
	public void fillApplicationsMenu(JMenu theMenu) {
		File startFolder = new File("." + File.separator + Application.baseFolder() + File.separator);
		this.fillMenu(startFolder, Application.fileExtension(), theMenu);
	}

	/**
	 * @param startFolder
	 * @param theMenu
	 */
	void fillMenu(File startFolder, String fileExtension, JMenu theMenu) {
		File[] files = startFolder.listFiles();
		for (int i=0; i<files.length; i++) {
			File aFile = (File)files[i];
			String filename = aFile.getName();
			if (filename.equals("CVS")) continue;
			if (filename.equals(".svn")) continue;
			if (aFile.isDirectory()) {
				JMenu aMenu = new JMenu(aFile.getName());
				theMenu.add(aMenu);
				this.fillMenu(aFile, fileExtension, aMenu);
				continue;
			}
			String shortFilename = filename.substring(0, filename.length()-4);
			String extension = filename.substring(filename.length()-4, filename.length());
			if (extension.equals(fileExtension)) {
				JMenuItem aMenuItem = new JMenuItem(shortFilename);
				if (fileExtension.equals(Application.fileExtension())) {
					applicationFiles.put(shortFilename, aFile.getAbsolutePath());
					this.addAddApplicationMenuActionListener(aMenuItem);
				} 
				if (fileExtension.equals(OperationsBox.fileExtension())) {
					operationFiles.put(shortFilename, aFile.getAbsolutePath());
					this.addAddOperationsMenuActionListener(aMenuItem);
				}
				theMenu.add(aMenuItem);
			}
		}
		
	}

	/**
	 * @param menuItem
	 */
	private void addAddOperationsMenuActionListener(JMenuItem menuItem) {
		menuItem.addActionListener(new java.awt.event.ActionListener() { 
			public void actionPerformed(java.awt.event.ActionEvent e) {    
				String aFilename = e.getActionCommand();
				String longName = (String)operationFiles.get(aFilename);
				OperationsBox newBox = OperationsBox.load(longName);
				newBox.show();
			}
		});
		
	}

	/**
	 * @param menuItem
	 */
	private void addAddApplicationMenuActionListener(JMenuItem menuItem) {
		menuItem.addActionListener(new java.awt.event.ActionListener() { 
			public void actionPerformed(java.awt.event.ActionEvent e) {    
				String aFilename = e.getActionCommand();
				String longName = (String)applicationFiles.get(aFilename);
				Application newApp = Application.load(longName);
				newApp.show();
			}
		});
	}

	/**
	 * @param menu6
	 */
	public void fillOperationsMenu(JMenu theMenu) {
		File startFolder = new File("." + File.separator + OperationsBox.baseFolder() + File.separator);
		this.fillMenu(startFolder, OperationsBox.fileExtension(), theMenu);
	}
	/**
	 * @return Returns the current.
	 */
	public static CellImageAnalyzer getCurrent() {
		if (current==null) current = new CellImageAnalyzer();
		return current;
	}
	/**
	 * @param current The current to set.
	 */
	public static void setCurrent(CellImageAnalyzer anApp) {
		current = anApp;
	}

	/**
	 * 
	 */
	public void rebuildOperationMenu() {
		view.rebuildOperationsMenu();
	}

	/**
	 * 
	 */
	public void rebuildApplicationMenu() {
		view.rebuildApplicationMenu();	
	}

	/**
	 * 
	 */
	public void toggleImageJ() {
		if (NJ.ntb!=null) return;
		if (IJ.getInstance().isShowing()) {
			IJ.getInstance().setVisible(false);
		} else {
			//IJ.getInstance().setBounds(10,10,100,20);
			IJ.getInstance().setVisible(true);
		}
		this.resetScale();
	}

	/**
	 * 
	 */
	public void startNeuronJ() {
		if (NJ.ntb!=null) return;
		IJ.getInstance().setVisible(false);
		NeuronJ_ neuronJ = new NeuronJ_();
		neuronJ.run("");
		(new NeuronJ()).show();
	}

	/**
	 * 
	 */
	public void substractMean() {
		ImagePlus inputImage = WindowManager.getCurrentImage();
		SubstractMeanOperation action = new SubstractMeanOperation();
		action.setInputImage(inputImage);
		action.execute();
	}

	/**
	 * 
	 */
	public void dilate() {
		ImagePlus inputImage = WindowManager.getCurrentImage();
		DilateOperation action = new DilateOperation();
		action.setInputImage(inputImage);
		action.execute();
	}

	/**
	 * 
	 */
	public void erode() {
		ImagePlus inputImage = WindowManager.getCurrentImage();
		ErodeOperation action = new ErodeOperation();
		action.setInputImage(inputImage);
		action.execute();
	}

	/**
	 * 
	 */
	public void start3DObjectCounter() {
		_3D_objects_counter counter = new _3D_objects_counter();
		gui.Executer executer = new gui.Executer(counter, "");
		Thread thread = new Thread(executer);
		thread.setPriority(Math.max(thread.getPriority()-2, Thread.MIN_PRIORITY));
		thread.start();
	}

	/**
	 * 
	 */
	public void startExportAsQuicktime() {
		MRI_QT_Stack_Writer_fast writer = new MRI_QT_Stack_Writer_fast();
		gui.Executer executer = new gui.Executer(writer, "");
		Thread thread = new Thread(executer);
		thread.setPriority(Math.max(thread.getPriority()-2, Thread.MIN_PRIORITY));
		thread.start();
	}

	/**
	 * @param files
	 */
	public void loadOperationCollections(File[] files) {
		for(int i=0; i<files.length; i++) {
			File file = files[i];
			OperationsBox opBox = OperationsBox.load(file.getAbsolutePath());
			opBox.show();
		}
	}

	
	/**
	 * @param files
	 */
	public void loadApplications(File[] files) {
		for(int i=0; i<files.length; i++) {
			File file = files[i];
			Application app = Application.load(file.getAbsolutePath());
			app.show();
		}
	}

	public CellImageAnalyzerView getView() {
		return view;
	}
	

    public void startInVivo() {
        IJ.doCommand("StkImage ");
    }
}
