/*
 * Created on 28.11.2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package gui;

import fileOperations.OpenImageOperation;
import ij.ImagePlus;
import ij.Prefs;
import ij.gui.PointRoi;
import ij.gui.PolygonRoi;
import ij.gui.Roi;
import ij.gui.Wand;
import java.awt.Color;
import java.awt.Polygon;
import java.io.IOException;
import java.io.Writer;
import java.util.Iterator;
import java.util.Vector;

import operations.Operation;
import patternMatching.CentralMomentFeatureCalculator;
import patternMatching.FeatureCalculator;
import patternMatching.FeatureVector;
import roiOperations.TransferSelectionOperation;

import analysisOperations.FindObjectsOperation;
import applications.Application;

/**
 * @author Volker
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class ObjectClass {
	PointRoi selection = new PointRoi(new int[0], new int[0], 0);
	String fileName = null;
	String name;
	Color color;
	ImagePlus mask;
	ImagePlus inputImage;
	private FeatureCalculator featureCalculator;
	FeatureVector[] features;
	
	public ObjectClass(String name) {
		this.name = name;
	}
	
	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}
	/**
	 * @param name The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}
	/**
	 * @return Returns the selections.
	 */
	public Roi getSelection() {
		return selection;
	}
	
	/**
	 * @param roi
	 * @param imageFile
	 */
	public void setSelection(PointRoi roi, String fileName) {
		this.fileName = fileName;
		selection = roi;
		// to do, each point has to be added only once
	}
	/**
	 * @return Returns the color.
	 */
	public Color getColor() {
		if (color==null) {
			color = Prefs.getColor(Prefs.ROICOLOR,Color.yellow);
		}
		return color;
	}
	/**
	 * @param color The color to set.
	 */
	public void setColor(Color color) {
		this.color = color;
	}

	/**
	 * @return
	 */
	public boolean isEmpty() {
		return (this.getSelection().getPolygon().npoints==0);
	}

	/**
	 * @param output
	 * @throws IOException
	 */
	public void putOn(Writer output) throws IOException {
		output.write(this.getName() + "\n");
		output.write(this.getSelection().getPolygon().npoints + "\n");
		float[] rgb = color.getRGBColorComponents(null);
		output.write(rgb[0] + ", ");
		output.write(rgb[1] + ", ");
		output.write(rgb[2] + "\n");
		output.write(this.fileName + "\n");
		for (int i=0; i<selection.getPolygon().npoints; i++) {
			output.write(selection.getPolygon().xpoints[i] + "," + selection.getPolygon().ypoints[i] + "\n");
		}
		for (int i=0; i<this.getFeatures().length; i++) {
			FeatureVector current = this.getFeatures()[i];
			current.putOn(output);
		}
	}
	/**
	 * @return Returns the fileName.
	 */
	public String getFileName() {
		return fileName;
	}
	/**
	 * @param fileName The fileName to set.
	 */
	public void setFileName(String fileName) {
		this.fileName = fileName;
	}
	
	public void calculateFeatures() {
		Polygon points = selection.getPolygon();
		ImagePlus aMask = this.getMask();
		Wand wand = new Wand(aMask.getProcessor());
		TransferSelectionOperation transferOp = new TransferSelectionOperation();
		FeatureCalculator featureCalc = this.getFeatureCalculator();
		this.features = new FeatureVector[points.npoints];
		for(int i=0; i<points.npoints; i++) {
			int x = points.xpoints[i];
			int y = points.ypoints[i];
			wand.autoOutline(x,y,255,255);
			if (wand.npoints>0) {
				Roi roi = new PolygonRoi(wand.xpoints, wand.ypoints, wand.npoints, Roi.TRACED_ROI);
				this.getInputImage().setRoi(roi); 
			}
			featureCalc.setInputImage(this.getInputImage());
			featureCalc.run();
			features[i] = featureCalc.getResult();
		}
	}
	/**
	 * @return
	 */
	private FeatureCalculator getFeatureCalculator() {
		if (this.featureCalculator==null) {
			this.setFeatureCalculator(this.getDefaultFeatureCalculator()); 
		}
		return featureCalculator;
	}

	/**
	 * @param defaultFeatureCalculator
	 */
	private void setFeatureCalculator(FeatureCalculator aFeatureCalculator) {
		this.featureCalculator = aFeatureCalculator;
		
	}

	/**
	 * @return
	 */
	private FeatureCalculator getDefaultFeatureCalculator() {
		FeatureCalculator result = new CentralMomentFeatureCalculator();
		return result;
	}

	/**
	 * 
	 */
	private void createMask() {
		Application app = Application.load("./_applications/particle counting/select all cells preprocessing.cia");
		app.removeOperation((Operation)app.getOperations().lastElement());
		Operation first = (Operation) app.getOperations().firstElement();
		first.setInputImage(this.getInputImage());
		app.run();
		FindObjectsOperation lastOp = (FindObjectsOperation)(app.getOperations().lastElement());
		mask = lastOp.getMask();
	}

	/**
	 * @return Returns the inputImage.
	 */
	public ImagePlus getInputImage() {
		if (inputImage==null) {
			loadInputImage();
		}
		return inputImage;
	}
	/**
	 * 
	 */
	private void loadInputImage() {
		OpenImageOperation op = new OpenImageOperation();
		op.setPath(this.getFileName());
		op.setShowResult(false);
		op.run();
		this.inputImage = op.getResult();
	}

	/**
	 * @param inputImage The inputImage to set.
	 */
	public void setInputImage(ImagePlus inputImage) {
		this.inputImage = inputImage;
	}
	/**
	 * @return Returns the mask.
	 */
	public ImagePlus getMask() {
		if (mask==null) {
			createMask();
		}
		return mask;
	}
	/**
	 * @param mask The mask to set.
	 */
	public void setMask(ImagePlus mask) {
		this.mask = mask;
	}
	/**
	 * @return Returns the features.
	 */
	public FeatureVector[] getFeatures() {
		if (features==null) {
			features = new FeatureVector[0];
		}
		return features;
	}

	/**
	 * @param comparisonObjectClasses
	 * @param distance
	 */
	public ObjectClass newClassWithCellsSimilarTo(Vector comparisonObjectClasses, double distance, String newName) {
		ObjectClass newClass = new ObjectClass(newName);
		PointRoi newRoi = null;
		FeatureVector[] allObjectFeatures = this.getFeatures();
		for (int i=0; i<allObjectFeatures.length; i++) {
			FeatureVector currentFeatureVector = allObjectFeatures[i];
			Iterator it = comparisonObjectClasses.iterator();
			while (it.hasNext()) {
				ObjectClass currentObjectClass = (ObjectClass) it.next();
				double currentDistance = currentObjectClass.distanceTo(currentFeatureVector);
				if (currentDistance <= distance) {
					int x = selection.getPolygon().xpoints[i];
					int y = selection.getPolygon().ypoints[i];
					if (newRoi!=null) 
						newRoi = newRoi.addPoint(x,y);
					else {
						int[] xComp = new int[1];
						xComp[0] = x;
						int[] yComp = new int[1];
						yComp[0] = y;
						newRoi = new PointRoi(xComp, yComp, 1);
					}
				}
			}
		}
		newClass.setSelection(newRoi, this.getFileName());
		return newClass;
	}

	/**
	 * @param comparisonObjectClasses
	 */
	public double[] distancesTo(Vector comparisonObjectClasses) {
		FeatureVector[] allObjectFeatures = this.getFeatures();
		double[] distances = new double[allObjectFeatures.length];
		for (int i=0; i<allObjectFeatures.length; i++) {
			FeatureVector currentFeatureVector = allObjectFeatures[i];
			double minDistance = Double.MAX_VALUE;
			Iterator it = comparisonObjectClasses.iterator();
			while (it.hasNext()) {
				ObjectClass currentObjectClass = (ObjectClass) it.next();
				double currentDistance = currentObjectClass.distanceTo(currentFeatureVector);
				if (currentDistance<minDistance) minDistance = currentDistance;
				}
			distances[i] = minDistance;
			}
		return distances;
	}
	/**
	 * @param currentFeatureVector
	 * @return
	 */
	private double distanceTo(FeatureVector featureVector) {
		double minDistance = Double.MAX_VALUE;
		FeatureVector[] allObjectFeatures = this.getFeatures();
		for (int i=0; i<allObjectFeatures.length; i++) {
			FeatureVector currentFeatureVector = allObjectFeatures[i];
			double currentDistance = currentFeatureVector.distanceTo(featureVector);
			if (currentDistance<minDistance) minDistance = currentDistance;
		}
		return minDistance;
	}
}
