package imageOperations;

import ij.ImagePlus;
import ij.ImageStack;
import ij.WindowManager;
import gui.BooleanOption;
import gui.Options;
import operations.Operation;
import stack.Stack_Combiner;

public class CombineImagesOperation extends Operation {
	/**
	 * 
	 */
	private static final long serialVersionUID = 3628029706683937964L;
	protected BooleanOption combineVertically;
	protected ImagePlus secondInputImage;
	
	protected void initialize() throws ClassNotFoundException {
		super.initialize();
		optionsNames = new String[1];
		optionsNames[0] = "combine vertically";
		parameterTypes = new Class[2];
		parameterTypes[0] = Class.forName("ij.ImagePlus");
		parameterTypes[1] = Class.forName("ij.ImagePlus");
		parameterNames = new String[2];
		parameterNames[0] = "InputImage";
		parameterNames[1] = "SecondInputImage";
		resultTypes = new Class[1];
		resultTypes[0] = Class.forName("ij.ImagePlus");
		resultNames = new String[1];
		resultNames[0] = "Result";
	}
	
	protected void setupOptions() {
		options = new Options(); 
		options.setName(this.name() + " options");
		String[] optionsNames = this.getOptionsNames();
		this.combineVertically = new BooleanOption();
		this.combineVertically.setValue("true");
		this.combineVertically.setName(optionsNames[0]);
		this.combineVertically.setShortHelpText("choose the arrangement of the images");
		options.add(this.combineVertically);
	}
	
	public void connectOptions() {
		combineVertically = (BooleanOption) options.getOptions().elementAt(0);
	}

	public boolean getCombineVertically() {
		return combineVertically.getBooleanValue();
	}

	public void setCombineVertically(boolean combineVertically) {
		this.combineVertically.setValue(Boolean.toString(combineVertically)); 
	}
	
	public void doIt() {
		ImagePlus inputImage = this.getInputImage();
		ImagePlus secondInputImage = this.getSecondInputImage();
		result = this.getCopyOfOrReferenceTo(inputImage, inputImage.getTitle());
		ImagePlus tmp = 
			this.getCopyOfOrReferenceTo(secondInputImage, secondInputImage.getTitle());
		Stack_Combiner combiner = new Stack_Combiner();
		ImageStack combined;
		if (this.getCombineVertically()) 	
			combined = combiner.combineVertically(result.getStack(), tmp.getStack()); 
		else
			combined = combiner.combineHorizontally(result.getStack(), tmp.getStack());
		result = new ImagePlus("Combined Images", combined);
	}

	public ImagePlus getSecondInputImage() {
		if (secondInputImage==null) {
			secondInputImage = WindowManager.getCurrentImage();
		}
		return secondInputImage;
	}

	public void setSecondInputImage(ImagePlus secondInputImage) {
		this.secondInputImage = secondInputImage;
	}
	
	protected void cleanUpInput() {
		super.cleanUpInput();
		this.setSecondInputImage(null);	
	}
}
