package imageOperations;

import ij.ImagePlus;
import ij.gui.NewImage;
import ij.measure.ResultsTable;
import ij.process.ImageProcessor;
import operations.Operation;

public class DrawCubeOperation extends Operation {

	/**
	 * 
	 */
	private static final long serialVersionUID = 8808007166894484608L;
	
	ResultsTable cube;

	protected void initialize() throws ClassNotFoundException {
		super.initialize();
		parameterTypes = new Class[2];
		parameterTypes[0] = Class.forName("ij.ImagePlus");
		parameterTypes[1] = Class.forName("ij.measure.ResultsTable");
		parameterNames = new String[2];
		parameterNames[0] = "InputImage";
		parameterNames[1] = "Cube";
		resultTypes = new Class[1];
		resultTypes[0] = Class.forName("ij.ImagePlus");
		resultNames = new String[1];
		resultNames[0] = "Result";
	}
	
	public void doIt() {
		ImagePlus inputImage = this.getInputImage();
		result = NewImage.createByteImage(inputImage.getTitle() + " - best cube", 
										  inputImage.getWidth(),
										  inputImage.getHeight(),
										  inputImage.getNSlices(),
										  NewImage.FILL_BLACK);
		int x = (int)this.getCube().getValue("x", 0);
		int y = (int)this.getCube().getValue("y", 0);
		int z = (int)this.getCube().getValue("z", 0);
		int radius = (int)this.getCube().getValue("radius", 0);
		int value = 255;
		for(int k=z-radius; k<=z+radius; k++) {
			if (k<1 || k>result.getNSlices()) continue;
			ImageProcessor ip = result.getStack().getProcessor(k);
			for (int i=x-radius; i<=x+radius; i++) {
				for (int j=y-radius; j<=y+radius; j++) {
					ip.putPixel(i,j,value);
				}
			}
		}
		result.updateAndDraw();
	}

	public ResultsTable getCube() {
		return cube;
	}

	public void setCube(ResultsTable cube) {
		this.cube = cube;
	}
}
