// **********************************************************************
// COPYRIGHT NOTICE AND DISCLAIMER
//
// Copyright (c) Erik Meijering. All rights reserved.
//
// Permission to copy and use this software and accompanying
// documentation for educational, research, and not-for-profit
// purposes, without fee and without a signed licensing agreement,
// is hereby granted, provided that the above copyright notice, this
// paragraph and the following two paragraphs appear in all copies.
// The copyright holder is free to make upgraded or improved versions
// of the software available for a fee or commercially only.
//
// IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE TO ANY PARTY FOR
// DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// OF ANY KIND WHATSOEVER, ARISING OUT OF THE USE OF THIS SOFTWARE
// AND ITS DOCUMENTATION, EVEN IF HE HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
// THE COPYRIGHT HOLDER SPECIFICALLY DISCLAIMS ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE AND ACCOMPANYING DOCUMENTATION IS PROVIDED "AS IS".
// THE COPYRIGHT HOLDER HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
// SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
// **********************************************************************

package imagescience.images;

/** Contains 4D integer coordinate values. */
public final class Coordinates {

    /** The value of the x-coordinate. */
    public int x = 0;

    /** The value of the y-coordinate. */
    public int y = 0;

    /** The value of the z-coordinate. */
    public int z = 0;

    /** The value of the t-coordinate. */
    public int t = 0;

    /** Default constructor. Each of the coordinates is set to 0. */
    public Coordinates() {}

    /** One-dimensional constructor. The x-coordinate is set to the
        given value. The y-, z-, and t-coordinates are set to 0. */
    public Coordinates(final int x) {
	this.x = x;
    }

    /** Two-dimensional constructor. The x- and y-coordinates are set to
        the given values. The z- and t-coordinates are set to 0. */
    public Coordinates(final int x, final int y) {
	this.x = x;
	this.y = y;
    }

    /** Three-dimensional constructor. The x-, y-, and z-coordinates are
        set to the given values. The t-coordinate is set to 0. */
    public Coordinates(final int x, final int y, final int z) {
	this.x = x;
	this.y = y;
	this.z = z;
    }

    /** Four-dimensional constructor. The x-, y-, z-, and t-coordinates
        are set to the given values. */
    public Coordinates(final int x, final int y, final int z, final int t) {
	this.x = x;
	this.y = y;
	this.z = z;
	this.t = t;
    }

    /** Copy constructor. The x-, y-, z-, and t-coordinates are set to
        the argument coordinates. */
    public Coordinates(final Coordinates coords) {
	this.x = coords.x;
	this.y = coords.y;
	this.z = coords.z;
	this.t = coords.t;
    }

    /** Sets the x-, y-, z-, and t-coordinates to 0. */
    public void reset() { this.x = this.y = this.z = this.t = 0; }

    /** Sets the x-, y-, z-, and t-coordinates to those of the argument.

	@exception NullPointerException if the parameter is <tt>null</tt>.
     */
    public void set(final Coordinates coords) {
	this.x = coords.x;
	this.y = coords.y;
	this.z = coords.z;
	this.t = coords.t;
    }

    /** Returns a copy of the current <tt>Coordinates</tt> object. */
    public Coordinates duplicate() { return new Coordinates(x,y,z,t); }

    /** Indicates whether or not the argument coordinates are equal to
        the current coordinates.

	@return <tt>true</tt> if each of the current coordinates is
        equal to the corresponding parameter coordinate; <tt>false</tt>
        if this is not the case.

	@exception NullPointerException if the parameter is <tt>null</tt>.
    */
    public boolean equals(Coordinates coords) {
	if (x==coords.x && y==coords.y && z==coords.z && t==coords.t) return true;
	return false;
    }

}
