// **********************************************************************
// COPYRIGHT NOTICE AND DISCLAIMER
//
// Copyright (c) Erik Meijering. All rights reserved.
//
// Permission to copy and use this software and accompanying
// documentation for educational, research, and not-for-profit
// purposes, without fee and without a signed licensing agreement,
// is hereby granted, provided that the above copyright notice, this
// paragraph and the following two paragraphs appear in all copies.
// The copyright holder is free to make upgraded or improved versions
// of the software available for a fee or commercially only.
//
// IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE TO ANY PARTY FOR
// DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// OF ANY KIND WHATSOEVER, ARISING OUT OF THE USE OF THIS SOFTWARE
// AND ITS DOCUMENTATION, EVEN IF HE HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
// THE COPYRIGHT HOLDER SPECIFICALLY DISCLAIMS ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE AND ACCOMPANYING DOCUMENTATION IS PROVIDED "AS IS".
// THE COPYRIGHT HOLDER HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
// SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
// **********************************************************************

package imagescience.utilities;

import java.lang.System;

/** Provides methods to facilitate the indication of the total
    computation time consumed by a process. */
public final class Timer {

    private long lStartTime;

    /** Starts the timer. Should be called at the beginning of the
        process whose total computation time is to be measured. */
    public void start() {
	lStartTime = System.currentTimeMillis();
    }

    /** Returns the total time in seconds elapsed since the call to
        <code>start()</code>. Should be called at the end of a
        process. */
    public double stop() {
	return (System.currentTimeMillis() - lStartTime)/1000.0;
    }

    /** Uses method <tt>debug()</tt> of <tt>msg</tt> to print the total
        time elapsed since the call to <tt>start()</tt>. Should be
        called at the end of a process.

	@exception NullPointerException if <code>msg</code> is
	<code>null</code>.
    */
    public void stop(final Messenger msg) {
	final long lRunTime = System.currentTimeMillis() - lStartTime;
	final long lHours = lRunTime/3600000;
	final long lMinutes = (lRunTime%3600000)/60000;
	final long lMilliSeconds = (lRunTime%3600000)%60000;
	msg.debug("Finished in " +
		  lHours + "h " +
		  lMinutes + "m " +
		  lMilliSeconds/1000 + "s " +
		  lMilliSeconds%1000 + "ms");
    }

}
