package magicWand;

import operations.Operation;
import processingOperations.FindEdgesOperation;
import processingOperations.GaussianBlurOperation;
import segmentationOperations.GreyscaleWatershedOperation;
import utils.ImageDuplicater;
import ij.ImagePlus;
import ij.WindowManager;
import imageOperations.ConvertImageTypeOperation;
import imageOperations.ImageCalculationOperation;
import gui.Option;
import gui.Options;

public class Watershed implements MagicWandBackstoreImageCreator {
	Options options;
	Option correctionSmoothingRadius;
	Option smoothingRadius;

	public Watershed() {
		this.setupOptions();
	}
	
	protected void setupOptions() {
		options = new Options();
		correctionSmoothingRadius = new Option();
		correctionSmoothingRadius.setName("unsharp mask smoothing radius");
		correctionSmoothingRadius.setShortHelpText("set the radius of the smoothing filter for the unsharp mask");
		correctionSmoothingRadius.setMin(1);
		correctionSmoothingRadius.setValue(Integer.toString(60));
		options.add(correctionSmoothingRadius);
		smoothingRadius = new Option();
		smoothingRadius.setName("smoothing radius");
		smoothingRadius.setShortHelpText("set the radius of the gaussian blur filter");
		smoothingRadius.setMin(1);
		smoothingRadius.setValue(Integer.toString(9));
		options.add(smoothingRadius);
	}
	
	public void run() {
		WindowManager.getTempCurrentImage().killRoi();
		ConvertImageTypeOperation convert = new ConvertImageTypeOperation();
		configureAndRunOperation(convert);
		convert.getResult().setImage(convert.getResult().getProcessor().createImage());
		FindEdgesOperation edges = new FindEdgesOperation();
		configureAndRunOperation(edges);
		
		ImagePlus flatfield = ImageDuplicater.copyImage(edges.getResult(), "blurred");
		GaussianBlurOperation blur = new GaussianBlurOperation();
		blur.setInputImage(flatfield);
		blur.setRadius(correctionSmoothingRadius.getIntegerValue());
		configureAndRunOperation(blur);
		
		ImageCalculationOperation calc = new ImageCalculationOperation();
		calc.setInputImage(edges.getResult());
		calc.setSecondInputImage(flatfield);
		calc.setOperator("Subtract");
		configureAndRunOperation(calc);
		
		GreyscaleWatershedOperation op = new GreyscaleWatershedOperation();
		op.setSmoothingRadius(this.smoothingRadius.getIntegerValue());
		op.setInputImage(calc.getResult());
		configureAndRunOperation(op);
		
		WindowManager.setTempCurrentImage(op.getResult());
/*		String name = IJ.getImage().getTitle();
		IJ.run("8-bit");
		IJ.run("Find Edges");
		IJ.run("Duplicate...", "copy");
		String smoothOptionString = "radius=" + this.correctionSmoothingRadius.getIntegerValue();
		IJ.run("Gaussian Blur...", smoothOptionString);
		IJ.run("Image Calculator...", "image1=" + "\"" + name + "\"" + " operation=Subtract image2=copy create");
		smoothOptionString = "radius=" + this.smoothingRadius.getIntegerValue();
		IJ.run("Gaussian Blur...", smoothOptionString);
		GreyscaleWatershedOperation op = new GreyscaleWatershedOperation();
		op.setKeepSource(false);
		op.run();*/
	}

	private void configureAndRunOperation(Operation op) {
		op.setShowResult(false);
		op.setKeepSource(false);
		op.run();
	}

	public void showOptions() {
		this.options.view().setVisible(true);
	}

	public String name() {
		return "watershed";
	}

}
