//**********************************************************************
//COPYRIGHT NOTICE AND DISCLAIMER
//
//Copyright (c) Erik Meijering. All rights reserved.
//
//Permission to copy and use this software and accompanying
//documentation for educational, research, and not-for-profit
//purposes, without fee and without a signed licensing agreement,
//is hereby granted, provided that the above copyright notice, this
//paragraph and the following two paragraphs appear in all copies.
//The copyright holder is free to make upgraded or improved versions
//of the software available for a fee or commercially only.
//
//IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE TO ANY PARTY FOR
//DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
//OF ANY KIND WHATSOEVER, ARISING OUT OF THE USE OF THIS SOFTWARE
//AND ITS DOCUMENTATION, EVEN IF HE HAS BEEN ADVISED OF THE
//POSSIBILITY OF SUCH DAMAGE.
//
//THE COPYRIGHT HOLDER SPECIFICALLY DISCLAIMS ANY WARRANTIES,
//INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
//MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
//SOFTWARE AND ACCOMPANYING DOCUMENTATION IS PROVIDED "AS IS".
//THE COPYRIGHT HOLDER HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
//SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
//**********************************************************************
package neuronJgui;

import ij.IJ;
import ij.gui.GUI;

import java.awt.Button;
import java.awt.Choice;
import java.awt.Dialog;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Panel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

public final class ColorsDialog extends Dialog implements ActionListener, ItemListener {

    private final Choice typeChoice;
    private final Choice colorChoice;

    private final Button okayButton;
    private final Button cancelButton;

    // Builds the dialog for setting the type colors.
    ColorsDialog() {

	super(IJ.getInstance(),"NeuronJ: Recolor",true);
	final GridBagConstraints c = new GridBagConstraints();
	final GridBagLayout grid = new GridBagLayout();
	setLayout(grid);

	// Add choices:
        typeChoice = new Choice();
	final int nrtypes = NJ.types.length;
        for (int i=0; i<nrtypes; ++i) typeChoice.addItem(NJ.types[i]);
        typeChoice.select(0);
	typeChoice.addItemListener(this);
	c.gridx = 0; c.gridy++; c.gridwidth = 1;
        c.anchor = GridBagConstraints.EAST;
        c.insets = new Insets(20, 15, 5, 0);
        grid.setConstraints(typeChoice, c);
        add(typeChoice);

        colorChoice = new Choice();
	final int nrcolors = NJ.colors.length;
        for (int i=0; i<nrcolors; ++i) colorChoice.addItem(NJ.colornames[i]);
        colorChoice.select(NJ.colorIndex(NJ.typecolors[0]));
        c.gridx = 1; c.gridwidth = 2;
        c.anchor = GridBagConstraints.WEST;
	c.insets = new Insets(20, 10, 5, 13);
        grid.setConstraints(colorChoice, c);
        add(colorChoice);

	// Add Okay and Cancel buttons:
	final Panel buttons = new Panel();
        buttons.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 0));
        okayButton = new Button("   OK   ");
        okayButton.addActionListener(this);
        cancelButton = new Button("Cancel");
        cancelButton.addActionListener(this);
        buttons.add(okayButton);
        buttons.add(cancelButton);
	c.gridx = 0; c.gridy++; c.gridwidth = 3;
        c.anchor = GridBagConstraints.EAST;
        c.insets = new Insets(15, 10, 12, 10);
        grid.setConstraints(buttons, c);
        add(buttons);

	// Pack and show:
	pack();
	GUI.center(this);
	setResizable(false);
	show();
    }

    public void actionPerformed(final ActionEvent e) {

	if (e.getSource() == okayButton) {
	    NJ.typecolors[typeChoice.getSelectedIndex()] = NJ.colors[colorChoice.getSelectedIndex()];
	    IJ.showStatus("Changed color");	    
	    NJ.nhd.redraw();
	    NJ.save = true;
	} else { IJ.showStatus(NJ.COPYRIGHT); }

	setVisible(false);
	dispose();
    }
	
    public void itemStateChanged(final ItemEvent e) {

        colorChoice.select(NJ.colorIndex(NJ.typecolors[typeChoice.getSelectedIndex()]));
    }

}
