//**********************************************************************
//COPYRIGHT NOTICE AND DISCLAIMER
//
//Copyright (c) Erik Meijering. All rights reserved.
//
//Permission to copy and use this software and accompanying
//documentation for educational, research, and not-for-profit
//purposes, without fee and without a signed licensing agreement,
//is hereby granted, provided that the above copyright notice, this
//paragraph and the following two paragraphs appear in all copies.
//The copyright holder is free to make upgraded or improved versions
//of the software available for a fee or commercially only.
//
//IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE TO ANY PARTY FOR
//DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
//OF ANY KIND WHATSOEVER, ARISING OUT OF THE USE OF THIS SOFTWARE
//AND ITS DOCUMENTATION, EVEN IF HE HAS BEEN ADVISED OF THE
//POSSIBILITY OF SUCH DAMAGE.
//
//THE COPYRIGHT HOLDER SPECIFICALLY DISCLAIMS ANY WARRANTIES,
//INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
//MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
//SOFTWARE AND ACCOMPANYING DOCUMENTATION IS PROVIDED "AS IS".
//THE COPYRIGHT HOLDER HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
//SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
//**********************************************************************
package neuronJgui;

import ij.gui.ImageCanvas;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;

public final class Segment {

    private int iCapacity = 500;
    private final int iCapInc = 500;
    private int iSize = 0;
    private int[] sx = null;
    private int[] sy = null;

    Segment() {
	sx = new int[iCapacity];
	sy = new int[iCapacity];
    }

    Segment(final int capacity) {
	iCapacity = capacity;
	sx = new int[iCapacity];
	sy = new int[iCapacity];
    }

    void add(final Point point) {
	if (iSize == iCapacity) inccap();
	sx[iSize] = point.x;
	sy[iSize++] = point.y;
    }
	
    private void inccap() {
	iCapacity += iCapInc;
	final int[] newarrayx = new int[iCapacity];
	final int[] newarrayy = new int[iCapacity];
	for (int i=0; i<iSize; ++i) {
	    newarrayx[i] = sx[i];
	    newarrayy[i] = sy[i];
	}
	sx = newarrayx;
	sy = newarrayy;
    }
	
    void element(final int index, final Point point) {
	point.x = sx[index];
	point.y = sy[index];
    }
	
    int nrpoints() { return iSize; }

    void reset() { iSize = 0; }

    Segment duplicate() {
	final Segment segment = new Segment(iCapacity);
	segment.iSize = iSize;
	for (int i=0; i<iSize; ++i) {
	    segment.sx[i] = sx[i];
	    segment.sy[i] = sy[i];
	}
	return segment;
    }

    double length() {
	double length = 0.0;
	if (iSize > 1) for (int i=1; i<iSize; ++i) {
	    final double dx = (sx[i] - sx[i-1])*NJ.xsize;
	    final double dy = (sy[i] - sy[i-1])*NJ.ysize;
	    length += Math.sqrt(dx*dx + dy*dy);
	}
	return length;
    }

    boolean nearby(final Point point) {
	final double NBR2 = NJ.NEARBYRANGE*NJ.NEARBYRANGE;
	for (int i=1, im1=0; i<iSize; ++i, ++im1) {
	    final double v12x = sx[i] - sx[im1];
	    final double v12y = sy[i] - sy[im1];
	    final double v13x = point.x - sx[im1];
	    final double v13y = point.y - sy[im1];
	    final double inprod = v12x*v13x + v12y*v13y;
	    if (inprod >= 0.0f) {
		final double v12len2 = v12x*v12x + v12y*v12y;
		if (inprod <= v12len2) {
		    final double v13len2 = v13x*v13x + v13y*v13y;
		    final double vrlen2 = inprod*inprod/v12len2;
		    if ((v13len2 - vrlen2) < NBR2) return true;
		}
	    }
	}
	return false;
    }

    void reverse() {
	final int iHalf = iSize/2;
	for (int b=0, e=iSize-1; b<iHalf; ++b, --e) {
	    final int tmpx = sx[b]; sx[b] = sx[e]; sx[e] = tmpx;
	    final int tmpy = sy[b]; sy[b] = sy[e]; sy[e] = tmpy;
	}
    }
	
    void draw(final Graphics g, final ImageCanvas imc, final Color color) {
	final Rectangle vof = imc.getSrcRect();
	final double mag = imc.getMagnification();
	final int dx = (int)(mag/2.0);
	final int dy = (int)(mag/2.0);
	g.setColor(color);
	if (iSize > 1) for (int i=1; i<iSize; ++i) {
	    g.drawLine(dx + (int)((sx[i] - vof.x)*mag),
		       dy + (int)((sy[i] - vof.y)*mag),
		       dx + (int)((sx[i-1] - vof.x)*mag),
		       dy + (int)((sy[i-1] - vof.y)*mag));
	}
    }

}
