//**********************************************************************
//COPYRIGHT NOTICE AND DISCLAIMER
//
//Copyright (c) Erik Meijering. All rights reserved.
//
//Permission to copy and use this software and accompanying
//documentation for educational, research, and not-for-profit
//purposes, without fee and without a signed licensing agreement,
//is hereby granted, provided that the above copyright notice, this
//paragraph and the following two paragraphs appear in all copies.
//The copyright holder is free to make upgraded or improved versions
//of the software available for a fee or commercially only.
//
//IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE TO ANY PARTY FOR
//DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
//OF ANY KIND WHATSOEVER, ARISING OUT OF THE USE OF THIS SOFTWARE
//AND ITS DOCUMENTATION, EVEN IF HE HAS BEEN ADVISED OF THE
//POSSIBILITY OF SUCH DAMAGE.
//
//THE COPYRIGHT HOLDER SPECIFICALLY DISCLAIMS ANY WARRANTIES,
//INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
//MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
//SOFTWARE AND ACCOMPANYING DOCUMENTATION IS PROVIDED "AS IS".
//THE COPYRIGHT HOLDER HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
//SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
//**********************************************************************
package neuronJgui;

import ij.ImagePlus;
import ij.gui.ImageCanvas;
import ij.gui.Roi;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

public final class VectorField extends Roi implements KeyListener {
	
    private ImagePlus imp;
    private ImageCanvas imc;
    private float[][][] vf;
    private int xsize;
    private int ysize;
    private float maxveclen = 1.0f;
    
    VectorField(final ImagePlus imp, final float[][][] vf) {

	super(0, 0, imp.getWidth(), imp.getHeight(), imp);
	this.imp = imp;
	this.imc = imp.getWindow().getCanvas();
	this.vf = vf;
	xsize = imp.getWidth();
	ysize = imp.getHeight();
	imp.setRoi(this);
	imc.addKeyListener(this);
    }

    public void draw(final Graphics g) {

	final float mag = (float)imc.getMagnification();
	if (mag > 4) {
	    final int dx = (int)(mag/2.0);
	    final int dy = (int)(mag/2.0);
	    g.setColor(Color.red);
//  	    g.setColor(Color.white);
	    final Rectangle vof = imc.getSrcRect();
	    final int xmax = vof.x + vof.width;
	    final int ymax = vof.y + vof.height;
	    for (int y=vof.y; y<ymax; ++y)
		for (int x=vof.x; x<xmax; ++x) {
		    final float scale = (255.0f - vf[0][y][x])*maxveclen*mag/255.0f;
		    final int hvx = (int)(vf[1][y][x]*scale)/2;
		    final int hvy = (int)(vf[2][y][x]*scale)/2;
		    g.drawLine(imc.screenX(x)-hvx+dx,imc.screenY(y)-hvy+dy,
			       imc.screenX(x)+hvx+dx,imc.screenY(y)+hvy+dy);
		}
	}
    }

    public void keyPressed(KeyEvent e) {

	if (e.getKeyCode() == KeyEvent.VK_UP) maxveclen += 0.05f;
	else if (e.getKeyCode() == KeyEvent.VK_DOWN) maxveclen -= 0.05f;

	imc.repaint();
    }

    public void keyReleased(KeyEvent e) {}

    public void keyTyped(KeyEvent e) {}

}
