package stackRegistration;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.IOException;

import ij.IJ;
import ij.ImagePlus;
import ij.io.FileInfo;
import ij.io.TiffDecoder;

public class AlignmentSlice extends Object {
	protected int xOffset = 0;
	protected int yOffset = 0;
	protected ImagePlus image;
	protected int width;
	protected int height;
	protected double score = 0.0d;
	protected String path;
	
	public AlignmentSlice(ImagePlus anImage) {
		image = anImage;
		image.setImage(image.getProcessor().createImage());
		width = anImage.getWidth();
		height = anImage.getHeight();
		this.path = anImage.getOriginalFileInfo().directory + File.separator + anImage.getOriginalFileInfo().fileName;
	}
	
	public AlignmentSlice(String path) {
		setPath(path);
		Point dims = readDimensions(path);
		this.width = dims.x;
		this.height = dims.y;
	}

	public void setPath(String path) {
		this.path = path;
	}
	
	public Point readDimensions(String path) {
		int width = 0;
		int height = 0;
		String directory = path.substring(0, path.lastIndexOf(File.separator));
		String filename = path.substring(path.lastIndexOf(File.separator), path.length());
		TiffDecoder decoder = new TiffDecoder(directory, filename);
		try {
			FileInfo info = decoder.getTiffInfo()[0];
  		    width = info.width;
  		    height = info.height;
		} catch (IOException e) {
			e.printStackTrace();
		}
		return new Point(width, height);
	}
	
	public AlignmentSlice() {
		// TODO Auto-generated constructor stub
	}

	public int getXOffset() {
		return xOffset;
	}
	
	public void setXOffset(int offset) {
		xOffset = offset;
	}
	
	public void setXOffset(Integer offset) {
		this.setXOffset(offset.intValue());
	}
	
	public void setYOffset(Integer offset) {
		this.setYOffset(offset.intValue());
	}
	
	public void setScore(Double score) {
		this.setScore(score);
	}
	
	public int getYOffset() {
		return yOffset;
	}
	
	public void setYOffset(int offset) {
		yOffset = offset;
	}
	
	public int getPixel(int x, int y) {
		return image.getPixel(x-xOffset, y-yOffset)[0];
	}
	
	public double alignSlice(AlignmentSlice theOtherSlice) {
		int x0 = xOffset-(width/2);
		int y0 = yOffset-(height/2);
		int bestX = x0;
		int bestY = y0;
		theOtherSlice.moveTo(x0, y0);
		double bestScore = this.calculateDifference(theOtherSlice);
		for (int i=1; i<width; i++) {
			for (int j=1; j<height; j++) {
				theOtherSlice.moveTo(x0+i, y0+j);
				double newScore = this.calculateDifference(theOtherSlice);
				if (newScore<bestScore) {
					bestScore = newScore;
					bestX = x0+i;
					bestY = y0+j;
				}
				IJ.showProgress((height)*(i-1)+j, width*height);
			}
		}
		theOtherSlice.moveTo(bestX, bestY);
		theOtherSlice.setScore(bestScore);
		return bestScore;
	}

	public double calculateDifference(AlignmentSlice theOtherSlice) {
		Rectangle image1Region = new Rectangle(this.xOffset, this.yOffset, this.width, this.height);
		Rectangle image2Region = new Rectangle(theOtherSlice.getXOffset(), theOtherSlice.getYOffset(), this.width, this.height);
		Rectangle2D overlapRegion = image1Region.createIntersection(image2Region);
		int startX = (int) Math.round(overlapRegion.getX());
		int startY = (int) Math.round(overlapRegion.getY());
		int endX = (int) Math.round(overlapRegion.getX() + overlapRegion.getWidth());
		int endY = (int) Math.round(overlapRegion.getY() + overlapRegion.getHeight());
		float result = 0;
		for (int i=startX; i<endX; i++) {
			for (int j=startY; j<endY; j++) {
				result += Math.abs(this.getPixel(i,j)-theOtherSlice.getPixel(i,j));
			}
		}
		result /= ((endX-startX) * (endY-startY) * 1.0d);
		return result;
	}

	public void moveTo(int i, int j) {
		this.xOffset = i;
		this.yOffset = j;
	}

	public int getHeight() {
		return height;
	}

	public void setHeight(int height) {
		this.height = height;
	}

	public ImagePlus getImage() {
		return image;
	}

	public void setImage(ImagePlus image) {
		this.image = image;
	}

	public double getScore() {
		return score;
	}

	public void setScore(double score) {
		this.score = score;
	}

	public int getWidth() {
		return width;
	}

	public void setWidth(int width) {
		this.width = width;
	}

	public String getTranslationString() {
		String result = "";
		result = result + this.getXOffset() + ",\t " + this.getYOffset() + " -\t " + this.getScore();
		return result;
	}

	public String getPath() {
		return path;
	}
}
