package stackRegistration;

import java.awt.Rectangle;
import java.awt.geom.Rectangle2D;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;

import magicWand.MagicWand;

import applications.Application;

import utils.ImageDuplicater;
import utils.ImageTransformer;

import fileOperations.SaveImageOperation;

import ij.IJ;
import ij.ImagePlus;
import ij.WindowManager;
import ij.gui.NewImage;
import imageOperations.ScaleImageOperation;
import imagejProxies.MRIInterpreter;

public class AlignmentSlice extends DisplaySlice {
	protected int xOffset = 0;
	protected int yOffset = 0;
	protected double score = 0.0d;
	protected Application application;
	
	public AlignmentSlice(ImagePlus anImage) {
		super(anImage);
	}
	
	public AlignmentSlice(String path) {
		super(path);
	}

	public AlignmentSlice() {
		// TODO Auto-generated constructor stub
	}

	public int getXOffset() {
		return xOffset;
	}
	
	public void setXOffset(int offset) {
		xOffset = offset;
	}
	
	public void setXOffset(Integer offset) {
		this.setXOffset(offset.intValue());
	}
	
	public void setYOffset(Integer offset) {
		this.setYOffset(offset.intValue());
	}
	
	public void setScore(Double score) {
		this.score = score.doubleValue();
	}
	
	public int getYOffset() {
		return yOffset;
	}
	
	public void setYOffset(int offset) {
		yOffset = offset;
	}
	
	public int getPixel(int x, int y) {
		return image.getPixel(x-xOffset, y-yOffset)[0];
	}
	
	public double alignSlice(AlignmentSlice theOtherSlice) {
		int x0 = xOffset-(width/2);
		int y0 = yOffset-(height/2);
		int bestX = x0;
		int bestY = y0;
		theOtherSlice.moveTo(x0, y0);
		double bestScore = this.calculateDifference(theOtherSlice);
		for (int i=1; i<width; i++) {
			for (int j=1; j<height; j++) {
				theOtherSlice.moveTo(x0+i, y0+j);
				double newScore = this.calculateDifference(theOtherSlice);
				if (newScore<bestScore) {
					bestScore = newScore;
					bestX = x0+i;
					bestY = y0+j;
				}
				IJ.showProgress((height)*(i-1)+j, width*height);
			}
		}
		theOtherSlice.moveTo(bestX, bestY);
		theOtherSlice.setScore(bestScore);
		return bestScore;
	}

	public double fastAlignSlice(AlignmentSlice theOtherSlice, int stepWidth) {
		int x0 = xOffset-(width/2);
		int y0 = yOffset-(height/2);
		int bestX = x0;
		int bestY = y0;
		theOtherSlice.moveTo(x0, y0);
		double bestScore = this.calculateDifference(theOtherSlice);
		for (int i=stepWidth; i<width; i+=stepWidth) {
			for (int j=stepWidth; j<height; j+=stepWidth) {
				theOtherSlice.moveTo(x0+i, y0+j);
				double newScore = this.calculateDifference(theOtherSlice);
				if (newScore<bestScore) {
					bestScore = newScore;
					bestX = x0+i;
					bestY = y0+j;
				}
				IJ.showProgress((height)*(i-1)+j, width*height);
			}
		}
		bestX = bestX-stepWidth;
		bestY = bestY-stepWidth;
		x0 = bestX;
		y0 = bestY;
		theOtherSlice.moveTo(x0, y0);
		for (int i=1; i<2*stepWidth; i++) {
			for (int j=1; j<2*stepWidth; j++) {
				theOtherSlice.moveTo(x0+i, y0+j);
				theOtherSlice.moveTo(x0+i, y0+j);
				double newScore = this.calculateDifference(theOtherSlice);
				if (newScore<bestScore) {
					bestScore = newScore;
					bestX = x0+i;
					bestY = y0+j;
				}
				IJ.showProgress((height)*(i-1)+j, width*height);
			}
		}
		theOtherSlice.moveTo(bestX, bestY);
		theOtherSlice.setScore(bestScore);
		return bestScore;
	}
	
	public double calculateDifference(AlignmentSlice theOtherSlice) {
		Rectangle image1Region = new Rectangle(this.xOffset, this.yOffset, this.width, this.height);
		Rectangle image2Region = new Rectangle(theOtherSlice.getXOffset(), theOtherSlice.getYOffset(), this.width, this.height);
		Rectangle2D overlapRegion = image1Region.createIntersection(image2Region);
		int startX = (int) Math.round(overlapRegion.getX());
		int startY = (int) Math.round(overlapRegion.getY());
		int endX = (int) Math.round(overlapRegion.getX() + overlapRegion.getWidth());
		int endY = (int) Math.round(overlapRegion.getY() + overlapRegion.getHeight());
		float result = 0;
		for (int i=startX; i<endX; i++) {
			for (int j=startY; j<endY; j++) {
				result += Math.abs(this.getPixel(i,j)-theOtherSlice.getPixel(i,j));
			}
		}
		result /= ((endX-startX) * (endY-startY) * 1.0d);
		return result;
	}

	public void moveTo(int i, int j) {
		this.xOffset = i;
		this.yOffset = j;
	}

	public double getScore() {
		return score;
	}

	public void setScore(double score) {
		this.score = score;
	}

	public String getTranslationString() {
		String result = "";
		result = result + this.getXOffset() + ",\t " + this.getYOffset() + " -\t " + this.getScore();
		return result;
	}

	public void saveWithSize(File targetFile, Rectangle zeroAndimageSize, boolean applyTranslations) {
		ImagePlus image = null;
		if (image==null) {
			image = this.loadImage();
		} else {
			image = this.getImage();
		}
		ImagePlus newImage;
		if (applyTranslations) {
			newImage = NewImage.createImage(image.getTitle(), zeroAndimageSize.width, zeroAndimageSize.height, 
												   image.getNSlices(), image.getBitDepth(), NewImage.FILL_BLACK);
		} else {
			newImage = ImageDuplicater.copyImage(image, image.getTitle());
		}
		MRIInterpreter interpreter = new MRIInterpreter();
		interpreter.setBatchMode(true);
		image.setImage(image.getProcessor().createImage());
		WindowManager.setTempCurrentImage(image);
		image.setRoi(this.getRoi());
		Application app = this.getApplication();
		if (app!=null) {
			app.run();
		}
		image.killRoi();
		ImagePlus result = image;
		WindowManager.setTempCurrentImage(image);
		interpreter.setBatchMode(true);
		if (applyTranslations) {
			IJ.run("Copy");
			newImage.setImage(newImage.getProcessor().createImage());
			WindowManager.setTempCurrentImage(newImage);
			IJ.run("Paste");
			IJ.run("Select None");
			result = ImageTransformer.shift(IJ.getImage(), this.getXOffset() - zeroAndimageSize.x, 
					this.getYOffset() - zeroAndimageSize.y);
		}
		SaveImageOperation save = new SaveImageOperation();
		save.setInputImage(result);
		save.setPath(targetFile.getParent() + File.separator + image.getOriginalFileInfo().fileName);
		save.run();
		newImage.changes = false;
		image.changes = false;
		this.setApplication(null);
		interpreter.setBatchMode(false);
 	}

	public void printOn(BufferedWriter out) {
		try {
			out.write(this.getXOffset() + "\t" + this.getYOffset() + "\t" + this.getScore() + "\n");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void setApplication(Application application) {
		this.application = application;
	}
	
	protected Application getApplication() {
		return application;
	}

	public void autoSelect(int i, int j) {
		MagicWand wand = MagicWand.getInstance();
		this.getImage().killRoi(); // test
		wand.doMagicWandAt(this.getImage(), i, j);
		this.setRoi(this.getImage().getRoi());
	}

	public void scale(File targetFile) {
		ScaleImageOperation operation = ((ScaleImageOperation)application.getOperations().firstElement());
		operation.setKeepSource(true);
		ImagePlus image = null;
		if (image==null) {
			image = this.loadImage();
		} else {
			image = this.getImage();
		}
		ImagePlus newImage = ImageDuplicater.copyImage(image, image.getTitle());
		MRIInterpreter interpreter = new MRIInterpreter();
		interpreter.setBatchMode(true);
		image.setImage(image.getProcessor().createImage());
		operation.setInputImage(newImage);
		Application app = this.getApplication();
		if (app!=null) {
			app.run();
		}
		ImagePlus result = operation.getResult();
		SaveImageOperation save = new SaveImageOperation();
		save.setInputImage(result);
		save.setPath(targetFile.getParent() + File.separator + image.getOriginalFileInfo().fileName);
		save.run();
		newImage.changes = false;
		image.changes = false;
		this.setApplication(null);
		interpreter.setBatchMode(false);
		double xScale = operation.getScaleFactorX();
		this.setXOffset((int)Math.round(this.getXOffset() * (xScale * 1.0)));
		double yScale = operation.getScaleFactorY();
		this.setYOffset((int)Math.round(this.getYOffset() * (yScale * 1.0)));
		this.setPath(targetFile.getParent() + File.separator + image.getOriginalFileInfo().fileName);
	}
}
