package stackRegistration;

import gui.CellImageAnalyzer;

import ij.ImagePlus;

import java.io.File;
import java.util.Iterator;
import java.util.Observable;
import java.util.Vector;

import operations.Operation;

public class ApplyOperationToSeries extends Observable {
	protected ApplyOperationToSeriesView view;
	protected Vector operations;
	protected ObjectModelingWorkbench registrator;
	protected boolean applyTranslations;
	
	public ApplyOperationToSeries(ObjectModelingWorkbench registrator) {
		super();
		this.registrator = registrator;
	}
	
	public void show() {
		this.getView().setVisible(true);
	}

	public ApplyOperationToSeriesView getView() {
		if (this.view==null) this.view = new ApplyOperationToSeriesView(this);
		return this.view;
	}

	public Vector getOperations() {
		if (this.operations==null) {
			this.setupOperations();
		}
		return operations;
	}

	public Vector getOperationNames() {
		Vector names = new Vector();
		Vector operations = this.getOperations();
		Iterator it = operations.iterator();
		while (it.hasNext()) {
			Operation currentOperation = (Operation) it.next();
				names.add(currentOperation.name());
		}
		return names;
	}
	
	protected void setupOperations() {
		Vector allOperations = CellImageAnalyzer.getCurrent().getOperationsFor("all").getOperations();
		operations = new Vector();
		Iterator it = allOperations.iterator();
		while (it.hasNext()) {
			Operation currentOperation = (Operation) it.next();
			Class[] parameterTypes = currentOperation.getParameterTypes();
			if (currentOperation.name().equals("close image")) continue;
			if (parameterTypes.length==1 && parameterTypes[0]==ImagePlus.class) {
				operations.add(currentOperation);
			}
		}
		
		
	}

	public void setOperations(Vector operations) {
		this.operations = operations;
	}

	public void applyOperationToSeries(Operation op, File targetFile) {
		registrator.applyOperationToSeries(op, targetFile, this.isApplyTranslations());
	}

	public boolean isApplyTranslations() {
		return applyTranslations;
	}

	public void setApplyTranslations(boolean b) {
		applyTranslations = b;
	}

	public void applyOperationToSlice(Operation op) {
		op.setKeepSource(false);
		op.run();
		// update display??
	}
}
