package stackRegistration;

import fileOperations.OpenImageOperation;
import gui.Option;
import ij.IJ;
import ij.ImagePlus;
import ij.measure.ResultsTable;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Vector;
import operations.Operation;

public class CalculateAlignmentTranslationsOperation extends Operation {

	protected Vector imageList;
	protected ResultsTable translations;
	protected String folder;
	protected BufferedWriter out;
	protected Option resultFilename;
	protected Option stepWidth;
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 779477588840854607L;
	
	public CalculateAlignmentTranslationsOperation() {
		super();
	}
	
	protected void initialize() throws ClassNotFoundException {
		super.initialize();
		parameterTypes = new Class[1];
		parameterTypes[0] = Class.forName("java.util.Vector");
		parameterNames = new String[1];
		parameterNames[0] = "ImageList";
		resultTypes = new Class[1];
		resultTypes[0] = Class.forName("ij.measure.ResultsTable");
		resultNames = new String[1];
		resultNames[0] = "Translations";
		optionsNames = new String[2];
		optionsNames[0] = "result filename";
		optionsNames[1] = "step width";
	}

	public Vector getImageList() {
		return imageList;
	}

	public void setImageList(Vector imageList) {
		this.imageList = imageList;
	}

	public ResultsTable getTranslations() {
		return translations;
	}

	public void setTranslations(ResultsTable translations) {
		this.translations = translations;
	}
	
	public void doIt() {
		boolean isFastAlignment = false;
		if (this.getStepWidth()>0) isFastAlignment = true;
		int stepWidth = this.getStepWidth();
		writeTimestamp();
		int counter = 0;
		OpenImageOperation open = new OpenImageOperation();
		String firstImageFileName = ((File)(this.getImageList().elementAt(counter))).getAbsolutePath();
		open.setAbsoluteFilename(firstImageFileName);
		open.setShowResult(false);
		open.run();
		if (this.resultFilename.getValue().equals("null") || this.resultFilename.getValue().trim().equals("")) {
			folder = firstImageFileName.substring(0, firstImageFileName.lastIndexOf(File.separator)) + File.separator;
			this.resultFilename.setValue(folder + "translations.txt");
		}
		try {
			out = new BufferedWriter(new FileWriter(this.resultFilename.getValue()));
		} catch (IOException e) {
			e.printStackTrace();
		}
		ImagePlus firstImage = open.getResult();
		AlignmentSlice firstSlice = new AlignmentSlice(firstImage);
		this.translations = new ResultsTable();
		this.translations.reset();
		translations.incrementCounter();
		translations.addValue(ResultsTable.X_CENTROID, 0);
		translations.addValue(ResultsTable.Y_CENTROID, 0);
		translations.addValue(ResultsTable.INTEGRATED_DENSITY, 0);
		counter++;
		this.writeToFile(0,0,0);
		while (counter<this.getImageList().size()) {
			System.gc();
			String secondImageFileName = ((File)(this.getImageList().elementAt(counter))).getAbsolutePath();
			open.setAbsoluteFilename(secondImageFileName);
			open.run();
			ImagePlus secondImage = open.getResult();
			AlignmentSlice secondSlice = new AlignmentSlice(secondImage);
			double score;
			if (isFastAlignment) {
				score = firstSlice.fastAlignSlice(secondSlice, stepWidth);
			} else {
				score = firstSlice.alignSlice(secondSlice);
			}
			translations.incrementCounter();
			translations.addValue(ResultsTable.X_CENTROID, secondSlice.getXOffset());
			translations.addValue(ResultsTable.Y_CENTROID, secondSlice.getYOffset());
			translations.addValue(ResultsTable.INTEGRATED_DENSITY, score);
			this.writeToFile(secondSlice.getXOffset(),secondSlice.getYOffset(),score);
			// prepare next loop cycle
			firstSlice = secondSlice;
			IJ.showProgress(counter, this.getImageList().size()-1);
			counter++;
			writeTimestamp();
		}
		try {
			out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		writeTimestamp();
	}

	private void writeTimestamp() {
		GregorianCalendar cal = new GregorianCalendar();
		System.out.println(cal.get(Calendar.HOUR_OF_DAY) + ":" + 
							cal.get(Calendar.MINUTE) + ":" + 
							cal.get(Calendar.SECOND) + "." + 
							cal.get(Calendar.MILLISECOND));
	}

	protected void writeToFile(int shiftX, int shiftY, double score) {
		try {
	        out.write(shiftX + "\t" + shiftY + "\t" + score + "\n");
	        out.flush();
	    } catch (IOException e) {
	    }
		
	}

	protected void setupOptions() {
		super.setupOptions();
		this.resultFilename.beForFilename();
		this.resultFilename.setShortHelpText("choose a result file for the translations");
		this.setStepWidth(0);
		this.stepWidth.setMin(0);
		this.stepWidth.setShortHelpText("if step width is >0 fast alignment is used");
		
	}
	
	public void connectOptions() {
		this.resultFilename = (Option) this.options.getOptions().elementAt(0);
		this.stepWidth = (Option) this.options.getOptions().elementAt(1);
	}

	public String getResultFilename() {
		return resultFilename.getValue();
	}

	public void setResultFilename(String filename) {
		this.resultFilename.setValue(filename);
	}

	public int getStepWidth() {
		return stepWidth.getIntegerValue();
	}

	public void setStepWidth(int stepWidth) {
		this.stepWidth.setValue(Integer.toString(stepWidth));
	}
}
