package stackRegistration;

import fileOperations.OpenImageOperation;
import ij.IJ;
import ij.ImagePlus;
import ij.measure.ResultsTable;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Vector;
import operations.Operation;
import sun.awt.shell.ShellFolder;

public class CalculateAlignmentTranslationsOperation extends Operation {

	protected Vector imageList;
	protected ResultsTable translations;
	protected String folder;
	protected BufferedWriter out;
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 779477588840854607L;
	
	protected void initialize() throws ClassNotFoundException {
		super.initialize();
		parameterTypes = new Class[1];
		parameterTypes[0] = Class.forName("java.util.Vector");
		parameterNames = new String[1];
		parameterNames[0] = "ImageList";
		resultTypes = new Class[1];
		resultTypes[0] = Class.forName("ij.measure.ResultsTable");
		resultNames = new String[1];
		resultNames[0] = "Translations";
	}

	public Vector getImageList() {
		return imageList;
	}

	public void setImageList(Vector imageList) {
		this.imageList = imageList;
	}

	public ResultsTable getTranslations() {
		return translations;
	}

	public void setTranslations(ResultsTable translations) {
		this.translations = translations;
	}
	
	public void doIt() {
		writeTimestamp();
		int counter = 0;
		OpenImageOperation open = new OpenImageOperation();
		String firstImageFileName = ((ShellFolder)(this.getImageList().elementAt(counter))).getAbsolutePath();
		open.setAbsoluteFilename(firstImageFileName);
		open.setShowResult(false);
		open.run();
		folder = firstImageFileName.substring(0, firstImageFileName.lastIndexOf(File.separator)) + File.separator;
		try {
			out = new BufferedWriter(new FileWriter(folder + "translations.txt"));
		} catch (IOException e) {
			e.printStackTrace();
		}
		ImagePlus firstImage = open.getResult();
		AlignmentSlice firstSlice = new AlignmentSlice(firstImage);
		this.translations = new ResultsTable();
		this.translations.reset();
		translations.incrementCounter();
		translations.addValue(ResultsTable.X_CENTROID, 0);
		translations.addValue(ResultsTable.Y_CENTROID, 0);
		translations.addValue(ResultsTable.INTEGRATED_DENSITY, 0);
		counter++;
		this.writeToFile(0,0,0);
		while (counter<this.getImageList().size()) {
			System.gc();
			String secondImageFileName = ((ShellFolder)(this.getImageList().elementAt(counter))).getAbsolutePath();
			open.setAbsoluteFilename(secondImageFileName);
			open.run();
			ImagePlus secondImage = open.getResult();
			AlignmentSlice secondSlice = new AlignmentSlice(secondImage);
			double score = firstSlice.alignSlice(secondSlice);
			translations.incrementCounter();
			translations.addValue(ResultsTable.X_CENTROID, secondSlice.getXOffset());
			translations.addValue(ResultsTable.Y_CENTROID, secondSlice.getYOffset());
			translations.addValue(ResultsTable.INTEGRATED_DENSITY, score);
			this.writeToFile(secondSlice.getXOffset(),secondSlice.getYOffset(),score);
			// prepare next loop cycle
			firstSlice = secondSlice;
			IJ.showProgress(counter, this.getImageList().size()-1);
			counter++;
			writeTimestamp();
		}
		try {
			out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		writeTimestamp();
	}

	private void writeTimestamp() {
		GregorianCalendar cal = new GregorianCalendar();
		System.out.println(cal.get(Calendar.HOUR_OF_DAY) + ":" + 
							cal.get(Calendar.MINUTE) + ":" + 
							cal.get(Calendar.SECOND) + "." + 
							cal.get(Calendar.MILLISECOND));
	}

	protected void writeToFile(int shiftX, int shiftY, double score) {
		try {
	        out.write(shiftX + "\t" + shiftY + "\t" + score + "\n");
	        out.flush();
	    } catch (IOException e) {
	    }
		
	}
	
}
