package stackRegistration;

import fileOperations.OpenImageOperation;
import ij.ImagePlus;
import ij.gui.Roi;
import ij.io.FileInfo;
import ij.io.TiffDecoder;

import java.awt.Point;
import java.io.File;
import java.io.IOException;

public class DisplaySlice {

	protected ImagePlus image;
	protected int width;
	protected int height;
	protected String path;
	protected ImageDisplayOptions displayOptions;
	protected Roi roi;

	public DisplaySlice() {
		super();
		// TODO Auto-generated constructor stub
	}

	public DisplaySlice(ImagePlus anImage) {
		image = anImage;
		image.setImage(image.getProcessor().createImage());
		width = anImage.getWidth();
		height = anImage.getHeight();
		this.path = anImage.getOriginalFileInfo().directory + File.separator + anImage.getOriginalFileInfo().fileName;
	}
	
	public DisplaySlice(String path) {
		setPath(path);
		Point dims = readDimensions(path);
		this.width = dims.x;
		this.height = dims.y;
	}
	
	public void setPath(String path) {
		this.path = path;
	}

	public Point readDimensions(String path) {
		int width = 0;
		int height = 0;
		String directory = path.substring(0, path.lastIndexOf(File.separator));
		String filename = path.substring(path.lastIndexOf(File.separator), path.length());
		TiffDecoder decoder = new TiffDecoder(directory, filename);
		try {
			FileInfo info = decoder.getTiffInfo()[0];
		    width = info.width;
		    height = info.height;
		} catch (IOException e) {
			e.printStackTrace();
		}
		return new Point(width, height);
	}

	public int getHeight() {
		return height;
	}

	public void setHeight(int height) {
		this.height = height;
	}

	public ImagePlus getImage() {
		return image;
	}

	public void setImage(ImagePlus image) {
		this.image = image;
	}

	public int getWidth() {
		return width;
	}

	public void setWidth(int width) {
		this.width = width;
	}

	public String getPath() {
		return path;
	}

	public ImagePlus loadImage() {
		OpenImageOperation open = new OpenImageOperation();
		open.setShowResult(false);
		open.setPath(this.getPath());
		open.run();
		open.getResult().setImage(open.getResult().getProcessor().createImage());
		return open.getResult();
	}

	public ImageDisplayOptions getDisplayOptions() {
		if (this.displayOptions==null) {
			this.displayOptions = new ImageDisplayOptions();
		}
		return displayOptions;
	}

	public void setDisplayOptions(ImageDisplayOptions displayOptions) {
		this.displayOptions = displayOptions;
	}

	public Roi getRoi() {
		return roi;
	}

	public void setRoi(Roi roi) {
		this.roi = roi;
	}

}
