package stackRegistration;

import ij.IJ;
import ij.ImagePlus;
import ij.gui.Roi;
import ij.gui.ShapeRoi;
import ij.util.Java2;
import java.awt.AlphaComposite;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.image.BufferStrategy;
import java.awt.image.BufferedImage;
import java.lang.reflect.Field;
import java.util.Iterator;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import javax.swing.JFrame;
import javax.swing.JPanel;

import utils.Broadcaster;

public class ImageCanvas extends ij.gui.ImageCanvas implements Observer {

	private static final long serialVersionUID = 3911048278762863753L;

	protected ImagePlus image;
	protected ImagePlus secondImage;
	protected int offsetX;
	protected int offsetY;
	protected int secondImageOffsetX;
	protected int secondImageOffsetY;
	public int tmpSecondImageOffsetX;
	public int tmpSecondImageOffsetY;	
	public int distance = 1;
	protected int scrollX = 0;
	protected int scrollY = 0;
	protected ImageDisplayOptions imageOptions;
	protected ImageDisplayOptions secondImageOptions;
	protected boolean bufferStrategySet = false;
	protected int width;
	protected int height;

	protected Broadcaster broadcaster;

	protected Object3DManager objectManager;

	public ImageCanvas() {
		super(new ImagePlus());
	}

	public void setImage(ImagePlus anImage) {
		image = anImage;
		updateImage(anImage);
	}

	void updateImage(ImagePlus imp) {
		this.imp = imp;
		int width = imp.getWidth();
		int height = imp.getHeight();
		imageWidth = width;
		imageHeight = height;
		srcRect = new Rectangle(0, 0, this.getWidth(), this.getHeight());
		setDrawingSize(this.getWidth(), this.getHeight());
		removeMouseListener(this);
		removeMouseMotionListener(this);
		removeKeyListener(IJ.getInstance());
		addMouseListener(this);
 		addMouseMotionListener(this);
 		addKeyListener(IJ.getInstance());
 		polygonRoiWorkaround();
 		if (imp!=null && imp.getWindow()!=null) this.setField(imp.getWindow(), "ic", this);
	}

	private void polygonRoiWorkaround() {
		srcRect.x = 1;
	}

	public void update(Graphics graphics) {
		BufferStrategy strategy = this.getBufferStrategy();
		Graphics bufferGraphics = strategy.getDrawGraphics();
		bufferGraphics.clearRect(0,0,this.getWidth()-1,this.getHeight()-1);
		super.update(bufferGraphics);
	}

	public void paint(Graphics graphics) {
		if (!bufferStrategySet) {
			bufferStrategySet = true;
			this.createBufferStrategy(2);
		}
		BufferStrategy strategy = this.getBufferStrategy();
		Graphics bufferGraphics = strategy.getDrawGraphics();
		bufferGraphics.translate(-scrollX, -scrollY);
		Roi roi = imp.getRoi();
		if (roi != null) {
			this.setField(imp.getRoi(), "ic", this );
			roi.updatePaste();
		}
		try {
			if (imageUpdated) {
				imageUpdated = false;
				imp.updateImage();
			}
			if (IJ.isJava2()) {
				if (magnification < 1.0)
					Java2.setBilinearInterpolation(graphics, true);
				else if (IJ.isMacOSX())
					Java2.setBilinearInterpolation(graphics, false);
			}
			drawImages(bufferGraphics);
			if (roi != null) {
				if (roi.getClass()== ShapeRoi.class) {
					fakeSourceRectForShapeRoi(roi);
				}
				roi.draw(bufferGraphics);
			}
			if (objectManager != null) drawObjects(bufferGraphics);
		} catch (OutOfMemoryError e) {
			IJ.outOfMemory("Paint");
		}
		bufferGraphics.dispose();
		strategy.show();
	}

	protected void fakeSourceRectForShapeRoi(Roi roi) {
		srcRect.x = (int)((offScreenX(-roi.getBounds().x)  + roi.getBounds().x / magnification)  - scrollX/magnification);
		srcRect.y = (int)((offScreenY(-roi.getBounds().y)  + roi.getBounds().y / magnification) - scrollY/magnification);
		srcRect.width = imp.getWidth();
		srcRect.height = imp.getHeight();
	}

	protected void drawImages(Graphics graphics) {
		if (image != null && imageOptions != null) {
			int zeroX = this.getParent().getWidth() / 2 - image.getWidth() / 2;
			int zeroY = this.getParent().getHeight() / 2 - image.getHeight() / 2;
			graphics.clearRect(0, 0, this.getWidth(), this.getHeight());
			drawOpaqueOrTransparent(graphics, image, zeroX + offsetX, zeroY +offsetY, imageOptions.getAlpha());
		}
		if (secondImage != null && secondImageOptions != null) {
			int zeroX = this.getParent().getWidth() / 2 - secondImage.getWidth() / 2;
			int zeroY = this.getParent().getHeight() / 2 - secondImage.getHeight() / 2;
			drawOpaqueOrTransparent(graphics, secondImage, 
										zeroX + secondImageOffsetX + tmpSecondImageOffsetX, 
										zeroY + secondImageOffsetY + tmpSecondImageOffsetY, 
										secondImageOptions.getAlpha());
		}
	}

	protected void drawObjects(Graphics graphics) {
		if (!objectManager.isShowObjects()) return;
		if (image != null && imageOptions != null) {
			int zeroX = this.getParent().getWidth() / 2 - image.getWidth() / 2;
			int zeroY = this.getParent().getHeight() / 2 - image.getHeight() / 2;
			int slice = objectManager.model.getView().getSelectionIndices()[0];
			Vector objects = objectManager.getObjectsForFirstSlice();
			Iterator it = objects.iterator();
			while (it.hasNext()) {
				Object3D object = (Object3D) it.next();
				if (!object.isVisible()) continue;
				this.drawObjectsOpaqueOrTransparent(graphics, object, zeroX + offsetX, zeroY +offsetY, slice);
			}
		}
		if (secondImage != null && secondImageOptions != null) {
			int zeroX = this.getParent().getWidth() / 2 - secondImage.getWidth() / 2;
			int zeroY = this.getParent().getHeight() / 2 - secondImage.getHeight() / 2;
			int slice = objectManager.model.getView().getSelectionIndices()[objectManager.model.getView().getSelectionIndices().length-1];
			Vector objects = objectManager.getObjectsForSecondSlice();
			Iterator it = objects.iterator();
			while (it.hasNext()) {
				Object3D object = (Object3D) it.next();
				if (!object.isVisible()) continue;
				this.drawObjectsOpaqueOrTransparent(graphics, object, 
						zeroX + secondImageOffsetX + tmpSecondImageOffsetX, 
						zeroY + secondImageOffsetY + tmpSecondImageOffsetY, slice);
			}
		}
	}
	
	protected void drawObjectsOpaqueOrTransparent(Graphics graphics, Object3D anObject, int x, int y, int slice) {
		Image theImage;
		Graphics theGraphics = graphics;
		theImage = createARGBObjectImageFor(anObject, slice);
		Graphics2D g2d = (Graphics2D) graphics;
		g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, anObject.getAlpha()));
		drawImage(theImage, theGraphics, x, y, image.getWidth(), image.getHeight());
	}

	protected void drawOpaqueOrTransparent(Graphics graphics, ImagePlus anImage, int x, int y, float alpha) {
		Image theImage = anImage.getImage();
		Graphics theGraphics = graphics;
		if (alpha < 1) {
			theImage = createARGBImageFor(anImage);
			Graphics2D g2d = (Graphics2D) graphics;
			g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, alpha));
		}
		drawImage(theImage, theGraphics, x, y, anImage.getWidth(), anImage.getHeight());
	}

	protected BufferedImage createARGBImageFor(ImagePlus anImage) {
		BufferedImage bi = new BufferedImage(
				anImage.getWidth(), anImage.getHeight(),
				BufferedImage.TYPE_INT_ARGB);
		Graphics g = bi.getGraphics();
		g.drawImage(anImage.getImage(), 0, 0, (int) (anImage.getWidth()), (int) (anImage.getHeight()), this);
		g.dispose();
		return bi;
	}

	protected Image createARGBObjectImageFor(Object3D anObject, int slice) {
		BufferedImage bi = new BufferedImage(
				image.getWidth(), image.getHeight(),
				BufferedImage.TYPE_INT_ARGB);
		Graphics2D g = (Graphics2D) bi.getGraphics();
		g.setRenderingHint(RenderingHints.KEY_ALPHA_INTERPOLATION,
		           RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY);
		g.setComposite(AlphaComposite.getInstance(
				 AlphaComposite.CLEAR, 0.0f));
		Roi aRoi = (Roi) anObject.getSelections().get(new Integer(slice));
		g.setComposite(AlphaComposite.SrcOver);
		g.setColor(anObject.getColor());
		Polygon aPolygon = aRoi.getPolygon();
		g.fillPolygon(aPolygon);
		g.dispose();
		return bi;
	}
	
	protected void drawImage(Image anImage, Graphics graphics, int x, int y, int width, int height) {
		graphics.drawImage(anImage, 
							x, y, (int) (x + (width * magnification)), (int) (y + (height * magnification)), 
							0, 0, width, height, 
						   this);
	}

	public void validateWindow() {
		JFrame win = (JFrame) (((JPanel) (this.getParent()))
				.getTopLevelAncestor());
		win.validate();
	}

	public void zoomIn(int x, int y) {
		super.zoomIn(x, y);
		polygonRoiWorkaround();
		this.validateWindow();
	}

	public void zoomOut(int x, int y) {
		super.zoomOut(x, y);
		polygonRoiWorkaround();
		this.validateWindow();
	}
	
	public void setXOffset(int offsetX) {
		this.offsetX = offsetX;
	}

	public void setYOffset(int offsetY) {
		this.offsetY = offsetY;
	}

	public int getScrollX() {
		return scrollX;
	}

	public void setScrollX(int scrollX) {
		this.scrollX = scrollX;
	}

	public int getScrollY() {
		return scrollY;
	}

	public void setScrollY(int scrollY) {
		this.scrollY = scrollY;
	}

	public ImagePlus getSecondImage() {
		return secondImage;
	}

	public void setSecondImage(ImagePlus secondImage) {
		this.secondImage = secondImage;
		tmpSecondImageOffsetX = 0;
		tmpSecondImageOffsetY = 0;
		if (secondImage==null) 
			this.updateImage(image); 
		else
			this.updateImage(secondImage);
	}

	public int getSecondImageOffsetX() {
		return secondImageOffsetX;
	}

	public void setSecondImageOffsetX(int secondImageOffsetX) {
		this.secondImageOffsetX = secondImageOffsetX;
	}

	public int getSecondImageOffsetY() {
		return secondImageOffsetY;
	}

	public void setSecondImageOffsetY(int secondImageOffsetY) {
		this.secondImageOffsetY = secondImageOffsetY;
	}

	public ImageDisplayOptions getImageOptions() {
		return imageOptions;
	}

	public void setImageOptions(ImageDisplayOptions imageOptions) {
		this.imageOptions = imageOptions;
		imageOptions.addObserver(this);
	}

	public ImageDisplayOptions getSecondImageOptions() {
		return secondImageOptions;
	}

	public void setSecondImageOptions(ImageDisplayOptions secondImageOptions) {
		this.secondImageOptions = secondImageOptions;
		secondImageOptions.addObserver(this);
	}

	public void update(Observable sender, Object aspect) {
		this.update(this.getGraphics());
	}
	
	/**Converts a screen x-coordinate to an offscreen x-coordinate.*/
	public int offScreenX(int x) {
		int offset = this.offsetX;
		if (secondImage!=null) {
			offset = this.secondImageOffsetX + tmpSecondImageOffsetX;
		}
		int result = scrollX;
		result = result + x;
		int zeroX = this.getParent().getWidth() / 2 - imp.getWidth() / 2;
		result = result - (zeroX + offset);
		result = (int)(result / magnification);
		return result; 
	}
		
	public int zeroX() {
		int zeroX = this.getParent().getWidth() / 2 - imp.getWidth() / 2;
		return zeroX;
	}
	
	public int zeroY() {
		int zeroY = this.getParent().getHeight() / 2 - imp.getHeight()/ 2;
		return zeroY;
	}
	/**Converts a screen y-coordinate to an offscreen y-coordinate.*/
	public int offScreenY(int y) {
		int offset = this.offsetY;
		if (secondImage!=null) {
			offset = this.secondImageOffsetY + tmpSecondImageOffsetY;
		}
		int result = scrollY;
		result = result + y;
		int zeroY = this.getParent().getHeight() / 2 - imp.getHeight()/ 2;
		result = result - (zeroY + offset);
		result = (int)(result / magnification);
		return result; 
	}
	
	public int screenX(int x) {
		double offset = this.offsetX;
		if (secondImage!=null) {
			offset = this.secondImageOffsetX + tmpSecondImageOffsetX;
		}
		double result = x*magnification;
		double zeroX = (this.getParent().getWidth() / 2  - image.getWidth() / 2) / magnification;
		result = result + zeroX*magnification + offset;
		return (int)result; 
	}
	
	public int screenY(int y) {
		double offset = this.offsetY;
		if (secondImage!=null) {
			offset = this.secondImageOffsetY + tmpSecondImageOffsetY;
		}
		double result = y * magnification;
		double zeroY = (this.getParent().getHeight() / 2 - image.getHeight() / 2) / magnification;
		result = result + zeroY * magnification + offset;
		return (int)result; 
	}
	
	public void mouseReleased(MouseEvent e) {
		if (this.offScreenX(e.getX())<0 || this.offScreenY(e.getY())<0) return;
		super.mouseReleased(e);
		getBroadcaster().changed("", "roi");
		if (e.isControlDown() && objectManager!=null) {
			objectManager.selectObjectAt(this.offScreenX(e.getX()), this.offScreenY(e.getY()), 
					objectManager.model.getView().getSelectionIndices()[objectManager.model.getView().getSelectionIndices().length-1]);
		}
		this.repaint();
	}
	
	public void mouseMoved(MouseEvent e) {
		super.mouseMoved(e);
		this.repaintRoi();
	}
	
	protected void repaintRoi() {
		if (imp==null) return;
		if (imp.getRoi()!=null) {
			Roi aRoi = imp.getRoi();
			if (aRoi.getState()==Roi.NORMAL) return;
			this.repaint();
		}
	}

	protected int getImageOriginX() {
		int offset = this.offsetX;
		if (secondImage!=null) {
			offset = this.secondImageOffsetX + tmpSecondImageOffsetX;
		}
		int zeroX = this.getParent().getWidth() / 2 - imp.getWidth() / 2;
		zeroX = zeroX + offset;
		zeroX = zeroX - scrollX;
		return zeroX;
	}

	protected int getImageOriginY() {
		int offset = this.offsetY;
		if (secondImage!=null) {
			offset = this.secondImageOffsetY + tmpSecondImageOffsetY;
		}
		int zeroY = this.getParent().getWidth() / 2 - imp.getHeight() / 2;
		zeroY = zeroY + offset;
		zeroY = zeroY - scrollY;
		return zeroY;
	}
	
	public void mouseDragged(MouseEvent e) {
		super.mouseDragged(e);
		this.repaintRoi();
	}
		
	public void mousePressed(MouseEvent e) {
		if (this.offScreenX(e.getX())<0 || this.offScreenY(e.getY())<0) return;
		super.mousePressed(e);
		this.repaintRoi();
	}
	
	public void setField(Object receiver, String name, Object value) {
		Class aClass = receiver.getClass();
		boolean found = false;
		while (aClass != Object.class) {
			try {
				found = true;
				Field theField = aClass.getDeclaredField(name);
				if (!found) continue;
				theField.setAccessible(true);
				theField.set(receiver, value);
				if (found) break;
			} catch (SecurityException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (NoSuchFieldException e) {
				found = false;
				aClass = aClass.getSuperclass();
			} catch (IllegalArgumentException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	public Object getField(Object receiver, String name) {
		Class aClass = receiver.getClass();
		boolean found = false;
		while (aClass != Object.class) {
			try {
				found = true;
				Field theField = receiver.getClass().getSuperclass().getDeclaredField(name);
				if (!found) continue;
				theField.setAccessible(true);
				Object result = theField.get(receiver);
				if (found) return result;
			} catch (SecurityException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (NoSuchFieldException e) {
				found = false;
				aClass = aClass.getSuperclass();
			} catch (IllegalArgumentException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
		}
		return null;
	}

	public int getHeight() {
		if (height==0) {
			height = this.getDefaultHeight();
		}
		return height;
	}

	public void setHeight(int height) {
		this.height = height;
		this.getBroadcaster().changed("height", Integer.toString(height));
		ImagePlus image = this.image;
		if (this.secondImage!=null) image = this.secondImage;
		this.updateImage(image);
		this.repaint();
		this.validateWindow();
	}

	public int getWidth() {
		if (width==0) {
			width = this.getDefaultWidth();
		}
		return width;
	}

	public void setWidth(int width) {
		this.width = width;
		this.getBroadcaster().changed("width", Integer.toString(width));
		ImagePlus image = this.image;
		if (this.secondImage!=null) image = this.secondImage;
		this.updateImage(image);
		this.repaint();
		this.validateWindow();
	}
	
	protected int getDefaultWidth() {
		return 2000;
	}

	protected int getDefaultHeight() {
		return 2000;
	}

	public Broadcaster getBroadcaster() {
		if (this.broadcaster==null) {
			this.broadcaster = new Broadcaster();
		}
		return broadcaster;
	}

	public void setObjectManager(Object3DManager objectManager) {
		this.objectManager = objectManager;
	}
}
