package stackRegistration;

import fileOperations.OpenImageOperation;
import gui.ListEditor;

import ij.ImagePlus;

import java.awt.Point;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Iterator;
import java.util.Observable;
import java.util.Vector;

import sun.awt.shell.ShellFolder;

public class ImageSeriesRegistrator extends Observable {
	protected Translations translations;
	protected ImagePlus firstImage;
	protected ImagePlus secondImage;
	protected int firstImageOffsetX;
	protected int firstImageOffsetY;
	protected int secondImageOffsetX;
	protected int secondImageOffsetY;

	/**
	 * @param string
	 */
	private void changed(String string) {
		this.setChanged();
		this.notifyObservers(string);
		this.clearChanged();
	}
	
	public void show() {
		ImageSeriesRegistratorView view = new ImageSeriesRegistratorView(this);
		view.setVisible(true);
	}
	
	public void openTranslations(String path) {
		Vector translations = new Vector();
		File aFile = new File(path);
		BufferedReader input = null;
		try {
			input = new BufferedReader(new FileReader(aFile));
			String line = null; 
			while ((line = input.readLine()) != null) {
				String[] items = line.split("\t");
				AlignmentSlice currentTranslation = new AlignmentSlice();
				currentTranslation.setXOffset(Integer.parseInt(items[0]));
				currentTranslation.setYOffset(Integer.parseInt(items[1]));
				currentTranslation.setScore(Double.parseDouble(items[2]));
				translations.add(currentTranslation);
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		this.translations.setTranslations(translations);
		this.changed("translations");
	}

	public Translations getTranslations() {
		if (translations==null) {
			translations = new Translations(new Vector());
		}
		return translations;
	}

	public void openSeries() {
		ListEditor editor = new ListEditor();
		editor.show();
		if (editor.getList().isEmpty()) {
			return;
		}
		this.openSeries(editor.getList());
	}
	
	public void openSeries(Vector list) {
		Iterator it = translations.getTranslations().iterator();
		int counter = 0;
		String firstPath = ((ShellFolder)list.elementAt(0)).getAbsolutePath();
		AlignmentSlice firstTranslation = (AlignmentSlice) translations.getTranslations().elementAt(0);
		Point dims = firstTranslation.readDimensions(firstPath);
		while (it.hasNext()) {
			AlignmentSlice translation = (AlignmentSlice) it.next();
			translation.setPath(((ShellFolder)list.elementAt(counter)).getAbsolutePath());
			translation.setWidth(dims.x);
			translation.setHeight(dims.y);
			counter++;
		}
	}

	public void loadImagesForSelection(Vector selection) {
		if (selection.size() > 0) {
			int index = ((Integer)selection.elementAt(0)).intValue();
			AlignmentSlice currentSlice = (AlignmentSlice)translations.getTranslations().elementAt(index);
			String filename = currentSlice.getPath();
			firstImageOffsetX = currentSlice.getXOffset();
			firstImageOffsetY = currentSlice.getYOffset();
			OpenImageOperation open = new OpenImageOperation();
			open.setShowResult(false);
			open.setPath(filename);
			open.run();
			this.firstImage = open.getResult();
			this.changed("image");
		}
		if (selection.size() > 1) {
			int index = ((Integer)selection.elementAt(selection.size()-1)).intValue();
			AlignmentSlice currentSlice = (AlignmentSlice)translations.getTranslations().elementAt(index);
			String filename = currentSlice.getPath();
			secondImageOffsetX = currentSlice.getXOffset();
			secondImageOffsetY = currentSlice.getYOffset();
			OpenImageOperation open = new OpenImageOperation();
			open.setShowResult(false);
			open.setPath(filename);
			open.run();
			this.secondImage = open.getResult();
			this.changed("secondImage");
		}
	}

	public ImagePlus getFirstImage() {
		return firstImage;
	}

	public int getFirstImageOffsetX() {
		return firstImageOffsetX;
	}

	public void setFirstImageOffsetX(int firstImageOffsetX) {
		this.firstImageOffsetX = firstImageOffsetX;
	}

	public int getFirstImageOffsetY() {
		return firstImageOffsetY;
	}

	public void setFirstImageOffsetY(int firstImageOffsetY) {
		this.firstImageOffsetY = firstImageOffsetY;
	}
	
	public ImagePlus getSecondImage() {
		return secondImage;
	}

	public void setSecondImage(ImagePlus secondImage) {
		this.secondImage = secondImage;
	}

	public int getSecondImageOffsetX() {
		return secondImageOffsetX;
	}

	public void setSecondImageOffsetX(int secondImageOffsetX) {
		this.secondImageOffsetX = secondImageOffsetX;
	}

	public int getSecondImageOffsetY() {
		return secondImageOffsetY;
	}

	public void setSecondImageOffsetY(int secondImageOffsetY) {
		this.secondImageOffsetY = secondImageOffsetY;
	}
}
