package stackRegistration;

import ij.IJ;
import ij.ImagePlus;
import ij.gui.ImageWindow;
import ij.io.OpenDialog;
import imagejProxies.MRIInterpreter;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableColumnModel;
import javax.swing.JViewport;
import javax.swing.JScrollBar;

public class ImageSeriesRegistratorView extends JFrame implements Observer {

	/**
	 * 
	 */
	private static final long serialVersionUID = -8439606539966828029L;
	private JPanel jContentPane = null;
	private JMenuBar jJMenuBar = null;
	private JMenu fileMenu = null;
	private JMenuItem openTranslationsMenuItem = null;
	private JMenuItem openStackMenuItem = null;
	
	protected ImageSeriesRegistrator model;
	private JScrollPane translationsTableScrollPane = null;
	private JTable translationsTable = null;
	private JViewport jViewport = null;
	private JPanel jPanel = null;
	private ImageCanvas imageCanvas = null;
	private JPanel scrollImagePanel = null;
	private JScrollBar vScrollBar = null;
	private JScrollBar hScrollBar = null;
	/**
	 * This is the default constructor
	 */
	public ImageSeriesRegistratorView() {
		super();
		initialize();
	}


	public ImageSeriesRegistratorView(ImageSeriesRegistrator model) {
		super();
		this.model = model;
		initialize();
		model.addObserver(this);
	}
	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(619, 401);
		this.setJMenuBar(getJJMenuBar());
		this.setContentPane(getJContentPane());
		this.setTitle("Image Series Registrator");
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJPanel(), java.awt.BorderLayout.WEST);
			jContentPane.add(getScrollImagePanel(), java.awt.BorderLayout.CENTER);
		}
		return jContentPane;
	}

	private ImageCanvas getImageCanvas() {
		if (imageCanvas==null) {
			imageCanvas = new ImageCanvas();
			imageCanvas.setImage(new ImagePlus());
			imageCanvas.setPreferredSize(new Dimension(2000,2000));
		}
		return imageCanvas;
	}


	/**
	 * This method initializes jJMenuBar	
	 * 	
	 * @return javax.swing.JMenuBar	
	 */
	private JMenuBar getJJMenuBar() {
		if (jJMenuBar == null) {
			jJMenuBar = new JMenuBar();
			jJMenuBar.add(getFileMenu());
		}
		return jJMenuBar;
	}

	/**
	 * This method initializes fileMenu	
	 * 	
	 * @return javax.swing.JMenu	
	 */
	private JMenu getFileMenu() {
		if (fileMenu == null) {
			fileMenu = new JMenu();
			fileMenu.setText("File");
			fileMenu.add(getOpenTranslationsMenuItem());
			fileMenu.add(getOpenStackMenuItem());
		}
		return fileMenu;
	}

	/**
	 * This method initializes jMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getOpenTranslationsMenuItem() {
		if (openTranslationsMenuItem == null) {
			openTranslationsMenuItem = new JMenuItem();
			openTranslationsMenuItem.setText("open translations...");
			openTranslationsMenuItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					OpenDialog.getDefaultDirectory();
					OpenDialog openDialog = new OpenDialog("Open translations", "translations", ".txt");
					String filename = openDialog.getFileName();
					if (filename==null) {
						return;
					}
					String folder = openDialog.getDirectory();
					String path = folder + filename;
					OpenDialog.setDefaultDirectory(folder);
					model.openTranslations(path);
				}
			});
		}
		return openTranslationsMenuItem;
	}

	/**
	 * This method initializes openStackMenuItem	
	 * 	
	 * @return javax.swing.JMenuItem	
	 */
	private JMenuItem getOpenStackMenuItem() {
		if (openStackMenuItem == null) {
			openStackMenuItem = new JMenuItem();
			openStackMenuItem.setText("open series...");
			openStackMenuItem.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					model.openSeries();
				}
			});
		}
		return openStackMenuItem;
	}


	public void update(Observable sender, Object aspect) {
		if (aspect.equals("translations")) {
			this.updateTranslations();
		}
		if (aspect.equals("image")) {
			this.updateImage();
		}
		if (aspect.equals("secondImage")) {
			this.updateSecondImage();
		}
	}


	protected void updateSecondImage() {
		if (this.getImageCanvas().secondImage!=null) {
			if (this.getImageCanvas().secondImage.getWindow()!=null) {
				this.getImageCanvas().secondImage.getWindow().close();
			}
		}
		ImagePlus image = model.getSecondImage();
		image.setImage(image.getProcessor().createImage());
		MRIInterpreter interpreter = new MRIInterpreter();
		interpreter.setBatchMode(true);
		ImageWindow win = new ImageWindow(image);
		image.setWindow(win);
		interpreter.setBatchMode(false);
		this.getImageCanvas().setSecondImage(image);
		this.getImageCanvas().setSecondImageOffsetX(model.getSecondImageOffsetX());
		this.getImageCanvas().setSecondImageOffsetY(model.getSecondImageOffsetY());
		this.getImageCanvas().validateWindow();
	}


	protected void updateImage() {
		this.getImageCanvas().setSecondImage(null);
		if (this.getImageCanvas().image!=null) {
			if (this.getImageCanvas().image.getWindow()!=null) {
				this.getImageCanvas().image.getWindow().close();
			}
		}
		ImagePlus image = model.getFirstImage();
		image.setImage(image.getProcessor().createImage());
		MRIInterpreter interpreter = new MRIInterpreter();
		interpreter.setBatchMode(true);
		ImageWindow win = new ImageWindow(image);
		image.setWindow(win);
		interpreter.setBatchMode(false);
		this.getImageCanvas().setImage(image);
		this.getImageCanvas().setXOffset(model.getFirstImageOffsetX());
		this.getImageCanvas().setYOffset(model.getFirstImageOffsetY());
		this.getImageCanvas().validateWindow();
	}


	protected void updateTranslations() {
		this.getTranslationsTable().repaint();
	}


	/**
	 * This method initializes translationsTableScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getTranslationsTableScrollPane() {
		if (translationsTableScrollPane == null) {
			translationsTableScrollPane = new JScrollPane();
			translationsTableScrollPane.setAlignmentX(0.0F);
			translationsTableScrollPane.setAlignmentY(0.0F);
			translationsTableScrollPane.setColumnHeader(getJViewport());
			translationsTableScrollPane.setViewportView(getTranslationsTable());
		}
		return translationsTableScrollPane;
	}


	/**
	 * This method initializes translationsTable	
	 * 	
	 * @return javax.swing.JTable	
	 */
	private JTable getTranslationsTable() {
		if (translationsTable == null) {
			translationsTable = new JTable(model.getTranslations());
			TableColumnModel tableColumnModel = translationsTable.getColumnModel();
			translationsTable.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF);
			translationsTable.setPreferredScrollableViewportSize(new Dimension(320, 70));
			tableColumnModel.getColumn(0).setWidth(20);
			tableColumnModel.getColumn(1).setWidth(25);
			tableColumnModel.getColumn(2).setWidth(30);
			tableColumnModel.getColumn(3).setWidth(40);
			ListSelectionModel rowSM = translationsTable.getSelectionModel();
			rowSM.addListSelectionListener(new ListSelectionListener() {
			    public void valueChanged(ListSelectionEvent e) {
			        //Ignore extra messages.
			        if (e.getValueIsAdjusting()) return;

			        ListSelectionModel lsm =
			            (ListSelectionModel)e.getSource();
			        if (lsm.isSelectionEmpty()) {
			            
			        } else {
			        	Vector selection = new Vector();
			        	for (int i=lsm.getMinSelectionIndex(); i<=lsm.getMaxSelectionIndex(); i++) {
			        		if (lsm.isSelectedIndex(i)) {
			        			selection.add(new Integer(i));
			        		}
			        	}
			        	model.loadImagesForSelection(selection);
			        }
			    }
			});
		}
		return translationsTable;
	}


	/**
	 * This method initializes jViewport	
	 * 	
	 * @return javax.swing.JViewport	
	 */
	private JViewport getJViewport() {
		if (jViewport == null) {
			jViewport = new JViewport();
			jViewport.setPreferredSize(new java.awt.Dimension(15,20));
		}
		return jViewport;
	}


	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel() {
		if (jPanel == null) {
			jPanel = new JPanel();
			jPanel.setLayout(new BorderLayout());
			jPanel.add(getTranslationsTableScrollPane(), java.awt.BorderLayout.CENTER);
		}
		return jPanel;
	}


	/**
	 * This method initializes scrollImagePanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getScrollImagePanel() {
		if (scrollImagePanel == null) {
			scrollImagePanel = new JPanel();
			scrollImagePanel.setLayout(new BorderLayout());
			scrollImagePanel.add(getImageCanvas(), java.awt.BorderLayout.CENTER);
			scrollImagePanel.add(getVScrollBar(), java.awt.BorderLayout.EAST);
			scrollImagePanel.add(getHScrollBar(), java.awt.BorderLayout.SOUTH);
		}
		return scrollImagePanel;
	}


	/**
	 * This method initializes vScrollBar	
	 * 	
	 * @return javax.swing.JScrollBar	
	 */
	private JScrollBar getVScrollBar() {
		if (vScrollBar == null) {
			vScrollBar = new JScrollBar();
			int height = this.getImageCanvas().getPreferredSize().height;
			vScrollBar.setMinimum(-height/2);
			vScrollBar.setMaximum(height/2);
			vScrollBar.setValue(0);
			vScrollBar.addAdjustmentListener(new java.awt.event.AdjustmentListener() {
				public void adjustmentValueChanged(java.awt.event.AdjustmentEvent e) {
					getImageCanvas().setScrollY(e.getValue());
					getImageCanvas().repaint();
					getImageCanvas().validateWindow();
				}
			});
		}
		return vScrollBar;
	}


	/**
	 * This method initializes hScrollBar	
	 * 	
	 * @return javax.swing.JScrollBar	
	 */
	private JScrollBar getHScrollBar() {
		if (hScrollBar == null) {
			hScrollBar = new JScrollBar();
			hScrollBar.setOrientation(javax.swing.JScrollBar.HORIZONTAL);
			int width = this.getImageCanvas().getPreferredSize().width;
			hScrollBar.setMinimum(-width/2);
			hScrollBar.setMaximum(width/2);
			hScrollBar.setValue(0);
			hScrollBar.addAdjustmentListener(new java.awt.event.AdjustmentListener() {
				public void adjustmentValueChanged(java.awt.event.AdjustmentEvent e) {
					getImageCanvas().setScrollX(e.getValue());
					getImageCanvas().repaint();
					getImageCanvas().validateWindow();
				}
			});
		}
		return hScrollBar;
	}
}  //  @jve:decl-index=0:visual-constraint="10,10"
