package stackRegistration;

import java.awt.BorderLayout;
import java.awt.event.KeyEvent;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.JLabel;
import java.awt.Toolkit;

public class ObjectModelingWorkbenchOptionsView extends JFrame implements Observer {

	/**
	 * 
	 */
	private static final long serialVersionUID = 8407861456297840823L;
	private JPanel jContentPane = null;
	private JTabbedPane jTabbedPane = null;
	private JPanel jPanel = null;
	private JTextField canvasWidthTextField = null;
	private JLabel canvasWidthLabel = null;
	private JLabel canvasHeightLabel = null;
	private JTextField canvasHeightTextField = null;
	protected ImageCanvas model;

	/**
	 * This is the default constructor
	 */
	public ObjectModelingWorkbenchOptionsView() {
		super();
		initialize();
	}
	
	public ObjectModelingWorkbenchOptionsView(ImageCanvas model) {
		super();
		this.model = model;
		model.getBroadcaster().addObserver(this);
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(300, 200);
		this.setIconImage(Toolkit.getDefaultToolkit().getImage(getClass().getResource("/icon.gif")));
		this.setContentPane(getJContentPane());
		this.setTitle("MRI OMW Options");
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJTabbedPane(), java.awt.BorderLayout.CENTER);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jTabbedPane	
	 * 	
	 * @return javax.swing.JTabbedPane	
	 */
	private JTabbedPane getJTabbedPane() {
		if (jTabbedPane == null) {
			jTabbedPane = new JTabbedPane();
			jTabbedPane.addTab("canvas size", null, getJPanel(), null);
		}
		return jTabbedPane;
	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel() {
		if (jPanel == null) {
			canvasHeightLabel = new JLabel();
			canvasHeightLabel.setText("canvas height:");
			canvasWidthLabel = new JLabel();
			canvasWidthLabel.setText("canvas width:");
			jPanel = new JPanel();
			jPanel.add(canvasWidthLabel, null);
			jPanel.add(getCanvasWidthTextField(), null);
			jPanel.add(canvasHeightLabel, null);
			jPanel.add(getCanvasHeightTextField(), null);
		}
		return jPanel;
	}

	/**
	 * This method initializes canvasWidthTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getCanvasWidthTextField() {
		if (canvasWidthTextField == null) {
			canvasWidthTextField = new JTextField();
			canvasWidthTextField.setPreferredSize(new java.awt.Dimension(50,20));
			canvasWidthTextField.setText(Integer.toString(model.getWidth()));
			canvasWidthTextField.addFocusListener(new java.awt.event.FocusAdapter() {
				public void focusLost(java.awt.event.FocusEvent e) {
					int value;
					try {
						value = Integer.parseInt(canvasWidthTextField.getText());
					} catch (Exception exc) {
						getCanvasWidthTextField().selectAll();
						getCanvasWidthTextField().grabFocus();
						return;
					}
					model.setWidth(value);
				}
			});
			canvasWidthTextField.addKeyListener(new java.awt.event.KeyAdapter() {
				public void keyPressed(java.awt.event.KeyEvent e) {
					if (e.getKeyCode()!=KeyEvent.VK_ENTER) return;
					int value;
					try {
						value = Integer.parseInt(canvasWidthTextField.getText());
					} catch (Exception exc) {
						getCanvasWidthTextField().selectAll();
						getCanvasWidthTextField().grabFocus();
						return;
					}
					model.setWidth(value);
				}
			});
		}
		return canvasWidthTextField;
	}

	/**
	 * This method initializes jTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getCanvasHeightTextField() {
		if (canvasHeightTextField == null) {
			canvasHeightTextField = new JTextField();
			canvasHeightTextField.setPreferredSize(new java.awt.Dimension(50,20));
			canvasHeightTextField.setText(Integer.toString(model.getHeight()));
			canvasHeightTextField.addFocusListener(new java.awt.event.FocusAdapter() {
				public void focusLost(java.awt.event.FocusEvent e) {
					int value;
					try {
						value = Integer.parseInt(canvasHeightTextField.getText());
					} catch (Exception exc) {
						getCanvasHeightTextField().selectAll();
						getCanvasHeightTextField().grabFocus();
						return;
					}
					model.setHeight(value);
				}
			});
			canvasHeightTextField.addKeyListener(new java.awt.event.KeyAdapter() {
				public void keyPressed(java.awt.event.KeyEvent e) {
					if (e.getKeyCode()!=KeyEvent.VK_ENTER) return;
					int value;
					try {
						value = Integer.parseInt(canvasHeightTextField.getText());
					} catch (Exception exc) {
						getCanvasHeightTextField().selectAll();
						getCanvasHeightTextField().grabFocus();
						return;
					}
					model.setHeight(value);
				}
			});
		}
		return canvasHeightTextField;
	}

	public void update(Observable aspect, Object value) {
		if (aspect.equals("width")) this.updateWidth((String)value);
		if (aspect.equals("height")) this.updateHeight((String)value);
	}

	protected void updateHeight(String height) {
		this.getCanvasHeightTextField().setText(height);
		
	}

	protected void updateWidth(String width) {
		this.getCanvasWidthTextField().setText(width);
	}

}
