package stackRegistration;

import java.awt.Point;
import java.awt.Rectangle;
import java.io.File;
import java.util.Iterator;
import java.util.Observer;
import java.util.Vector;

import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;

import applications.Application;

public class Translations extends AbstractTableModel implements TableModel {

	/**
	 * 
	 */
	private static final long serialVersionUID = -2690300221461261456L;
	protected Vector translations;
	protected Application application;
	
	public Translations() {
		super();
		// TODO Auto-generated constructor stub
	}

	public Translations(Vector theTranslations) {
		super();
		this.translations = theTranslations;
		this.fireTableRowsInserted(0, theTranslations.size()-1);
		this.fireTableDataChanged();
	}
	
	public int getRowCount() {
		return this.getTranslations().size();
	}

	public int getColumnCount() {
		return 4;
	}

	public String getColumnName(int index) {
		String[] columnNames = {"nr.", "x", "y", "score"};
		return columnNames[index];
	}

	public Class getColumnClass(int index) {
		Class[] classes = {Integer.class, Integer.class, Integer.class, Double.class};
		return classes[index];
	}

	public boolean isCellEditable(int rowIndex, int columnIndex) {
		if (columnIndex == 0) return false; 
		return true;
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		Object result = null;
		AlignmentSlice row = (AlignmentSlice) this.getTranslations().elementAt(rowIndex);
		if (columnIndex == 0) return new Integer(rowIndex+1);
		if (columnIndex == 1) return new Integer(row.getXOffset());
		if (columnIndex == 2) return new Integer(row.getYOffset());
		if (columnIndex == 3) return new Double(row.getScore());
		return result;
	}

	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		if (columnIndex==0) return;
		if (columnIndex==1) ((AlignmentSlice)(this.getTranslations().elementAt(rowIndex))).setXOffset((Integer)aValue);
		if (columnIndex==2) ((AlignmentSlice)(this.getTranslations().elementAt(rowIndex))).setYOffset((Integer)aValue);
		if (columnIndex==3) ((AlignmentSlice)(this.getTranslations().elementAt(rowIndex))).setScore((Double)aValue);
		fireTableCellUpdated(rowIndex, columnIndex);
	}

	public Vector getTranslations() {
		if (translations==null) {
			translations = new Vector();
		}
		return translations;
	}

	public void setTranslations(Vector translations) {
		this.translations = translations;
	//	this.fireTableRowsInserted(0, translations.size()-1);
		this.fireTableDataChanged();
	}

	public Rectangle calculateImageSize() {
		Point result = new Point(0,0);
		if (this.getTranslations().isEmpty()) return null;
		int minX=0, maxX=0, minY=0, maxY = 0;
		Iterator it = this.getTranslations().iterator();
		while(it.hasNext()) {
			AlignmentSlice currentTranslation = (AlignmentSlice)it.next();
			if (currentTranslation.getXOffset() < minX) minX = currentTranslation.getXOffset();
			if (currentTranslation.getXOffset() > maxX) maxX = currentTranslation.getXOffset();
			if (currentTranslation.getYOffset() < minY) minY = currentTranslation.getYOffset();
			if (currentTranslation.getYOffset() > maxY) maxY = currentTranslation.getYOffset();
		}
		int xAdd = 0;
		if (maxX*minX<0) xAdd += Math.abs(maxX) + Math.abs(minX);
		if (maxX*minX>=0) xAdd += Math.max(Math.abs(minX), Math.abs(maxX));
		int yAdd = 0;
		if (maxY*minY<0) yAdd += Math.abs(maxY) + Math.abs(minY);
		if (maxY*minY>=0) yAdd += Math.max(Math.abs(minY), Math.abs(maxY));
		int width = ((AlignmentSlice)(this.getTranslations().elementAt(0))).getWidth();
		int height =((AlignmentSlice)(this.getTranslations().elementAt(0))).getHeight();
		int xResult = width + xAdd;
		int yResult = height + yAdd;
		result.setLocation(xResult, yResult);
		Rectangle resultRectangle = new Rectangle(Math.round((minX + maxX) / 2.0f), 
												  Math.round((minY + maxY) / 2.0f),
												  xResult, yResult);
		return resultRectangle;
	}

	public void saveWithSize(File targetFile, Rectangle zeroAndimageSize, boolean applyTranslations) {
		TreatAndSaveSlices saver = new TreatAndSaveSlices(this.getTranslations(), targetFile, zeroAndimageSize, this.getApplication(), applyTranslations);
		saver.setShowProgress(true);
		saver.execute();
	}

	public void scale(File targetFile, Observer observer) {
		ScaleSlices scale = new ScaleSlices(this.getTranslations(), targetFile, this.getApplication());
		scale.setShowProgress(true);
		if (observer!=null) scale.addObserver(observer);
		scale.execute();
	}
	public Application getApplication() {
		return application;
	}

	public void setApplication(Application application) {
		this.application = application;
	}
}
